# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

# THIS IS Lernid CONFIGURATION FILE
# YOU CAN PUT THERE SOME GLOBAL VALUE
# Do not touch until you know what you're doing.
# you're warned :)

# where your project will head for your data (for instance, images and ui files)
# by default, this is ../data, relative your trunk layout
__lernid_data_directory__ = '../data/'


import os
import xdg.BaseDirectory

def _(message): return message

VERSION = '1.0.8'
DESCRIPTION = _('Connect to a world of online tutorials quickly and easily.')
WEBSITE = 'http://wiki.ubuntu.com/Lernid'
CONTRIBUTORS = [
    "Alex Kapranoff <kapranoff@gmail.com>",
    "Andrew Higginson <rugby471@gmail.com>",
    "Daniel Holbach <daniel.holbach@ubuntu.com>",
    "David Planella <david.planella@ubuntu.com>",
    "Jono Bacon <jono@ubuntu.com>",
    "Lucian Adrian Grijincu <lucian.grijincu@gmail.com>",
    "Methiue Comandon <strycore@gmail.com>",
    "Michael Budde <mbudde@gmail.com>",
    "Peeyoosh Sangolekar <piyush_sangolekar@hotmail.com>",
    "Sanne Wouda <sanne@gruttepier.net>",
    "Sense Hofstede <sense@qense.nl>",
    "John S Gruber <johnsgruber@gmail.com>",
]
ARTISTS = [
    'K.Vishnoo Charan Reddy <foo.mac.v@gmail.com>'
]

LICENSE = """
This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License version 3, as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful, but  WITHOUT ANY WARRANTY; without even the implied warranties of MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along  with this program.  If not, see <http://www.gnu.org/licenses/>.
"""

class ProjectPathNotFound(Exception):
    pass

def get_data_path(*resource):
    """Retrieve lernid data path

    This path is by default <lernid_lib_path>/../data/ in trunk
    and /usr/share/lernid in an installed version but this path
    is specified at installation time.
    """

    # get pathname absolute or relative
    if __lernid_data_directory__.startswith('/'):
        pathname = __lernid_data_directory__
    else:
        pathname = os.path.join(os.path.dirname(__file__), __lernid_data_directory__)
    abs_data_path = os.path.abspath(pathname)
    if os.path.exists(abs_data_path):
        filepath = os.path.join(abs_data_path, *resource)
        if os.path.exists(filepath):
            return filepath
        else:
            return None
    else:
        raise ProjectPathNotFound

# Taken from xdg.BaseDirectory.save_data_path() to provide the same function
# $XDG_CACHE_HOME as well
def save_cache_path(*resource):
    """Ensure $XDG_CACHE_HOME/<resource>/ exists, and return its path.
    'resource' is the name of some shared resource. Use this when updating
    a shared (between programs) database. Use the xdg_data_dirs variable
    for loading."""
    resource = os.path.join(*resource)
    assert not resource.startswith('/')
    path = os.path.join(xdg.BaseDirectory.xdg_cache_home, resource)
    if not os.path.isdir(path):
        os.makedirs(path)
    return path



#!/usr/bin/python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE


from ConfigParser import RawConfigParser
from lernid.lernidconfig import save_cache_path
from lernid.LernidOptions import Options
import logging


class Preferences(object):

    _instance = None

    def __init__(self):
        self._parser = RawConfigParser()
        self._parser.add_section('lernid')
        self._load_preferences()

    def _load_preferences(self):
        self._preferences = {
            'vertical': False,
            'show_appindicator': True,
            'main_window_maximize': False,
            'statusbar': True,
            'nick': '',
            'show_irc_time': False,
            'links_in_default_browser': False

        }
        file_name = save_cache_path('lernid')+'/config'
        try:
            config_file = open(file_name, 'r')
        except: 
            return
        self._parser.readfp(config_file)
        config_file.close()
        for item in self._parser.items('lernid'):
            if item[0] == 'width':
                if 'main_window_size' not in self._preferences:
                    self._preferences['main_window_size'] = dict()
                self._preferences['main_window_size']['width'] = int(item[1])
                continue
            if item[0] == 'height':
                if 'main_window_size' not in self._preferences:
                    self._preferences['main_window_size'] = dict()
                self._preferences['main_window_size']['height'] = int(item[1])
                continue
            if item[0] == 'hpanesmain':
                if 'hpanes' not in self._preferences:
                    self._preferences['hpanes'] = dict()
                self._preferences['hpanes']['main'] = int(item[1])
                continue
            if item[0] == 'hpanessub':
                if 'hpanes' not in self._preferences:
                    self._preferences['hpanes'] = dict()
                self._preferences['hpanes']['sub'] = int(item[1])
                continue
            if item[0] == 'vpanesmain':
                if 'vpanes' not in self._preferences:
                    self._preferences['vpanes'] = dict()
                self._preferences['vpanes']['main'] = int(item[1])
                continue
            if item[0] == 'vpanessub':
                if 'vpanes' not in self._preferences:
                    self._preferences['vpanes'] = dict()
                self._preferences['vpanes']['sub'] = int(item[1])
                continue
            if item[1].startswith('{'):        # remove old style unused options
                r = self._parser.remove_option('lernid', item[0])
                continue
            elif item[1] == "True":
                self._preferences[item[0]] = True
            elif item[1] == "False":
                self._preferences[item[0]] = False
            else:
                self._preferences[item[0]] = item[1]

    def _update(self, prefs):
        self._preferences.update(prefs)

    def _flush(self):
       for item in self._preferences:
           if item == 'hpanes':
               self._parser.set('lernid', 'hpanesmain',
                   self._preferences[item]['main'])
               self._parser.set('lernid', 'hpanessub',
                   self._preferences[item]['sub'])
           elif item == 'vpanes':
               self._parser.set('lernid', 'vpanesmain',
                   self._preferences[item]['main'])
               self._parser.set('lernid', 'vpanessub',
                   self._preferences[item]['sub'])
           elif item == 'main_window_size':
               self._parser.set('lernid', 'width',
                   self._preferences[item]['width'])
               self._parser.set('lernid', 'height',
                   self._preferences[item]['height'])
           else:
               self._parser.set('lernid', item, self._preferences[item])

       config_file = open(save_cache_path('lernid')+'/config', 'w')
       self._parser.write(config_file)
       config_file.close()

    @classmethod
    def get_instance(cls):
        if not cls._instance:
            cls._instance = cls()
        return cls._instance

    @classmethod
    def get(cls, name=None, default=None):
        """Return dict with preferences if @name is not given, else return
        the value of preference @name. If @name is not set, return @default
        which defaults to None.
        """
        klass = cls.get_instance()
        if name:
            return klass._preferences.get(name, default)
        return klass._preferences

    @classmethod
    def set(cls, *args, **kwargs):
        """set(dict) -> updates preferences with values from dict
        set(name, value) -> updates a single preference
        set(key=val, key=val, ...) -> updates preferences with key-value pairs
        """
        klass = cls.get_instance()
        if len(args) == 1:
            d = args[0]
        elif len(args) == 2:
            name, value = args
            d = {name: value}
        elif len(args) == 0 and kwargs:
            d = kwargs
        else:
            raise ValueError('set should be called as either set(key, val), '
                             'set(dict) or set(key=val, key=val, ...)')
        klass._update(d)

    @classmethod
    def flush(cls):
        klass = cls.get_instance()
        klass._flush()

if __name__ == '__main__':
    print Preferences.get('width')
    Preferences.set('width', 100)
    print Preferences.get('width')
    Preferences.set(width=200, height=300)
    print Preferences.get()
    Preferences.set({'width': 500, 'height': 200})
    print Preferences.get()



#!/usr/bin/python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
# Copyright (c) 2011 John S Gruber <johnsgruber@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import glib
import gobject
import gtk
import gio
import os
import time
import urlparse
import random
import pynotify
import logging
from datetime import timedelta

import lernid.DateTime as dt
from lernid.widgets.Widget import Widget
from lernid.lernidconfig import get_data_path
from lernid.Sessions import Session, parse_ical, read_ical, start_read_ical
from lernid.LernidOptions import Options


class Schedule(Widget):

    __gtype_name__ = 'LernidSchedule'

    __gsignals__ = {
        'session-changed': (
            gobject.SIGNAL_RUN_LAST, None, (object,)
        ),
        'session-ended': (
            gobject.SIGNAL_RUN_LAST, None, ()
        ),
    }

    COL_ICON, COL_DATE, COL_START, COL_END, COL_TITLE, COL_SESSION, \
    COL_DATETIME = range(7)

    def __init__(self):
        Widget.__init__(self, 'schedule')
        vbox = gtk.VBox()
        self.add(vbox)
        scroll = gtk.ScrolledWindow()
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        vbox.pack_start(scroll, expand=True)
        hbuttonbox = gtk.HBox(False, 0)
        vbox.pack_start(hbuttonbox, expand=False)
        self._copy_button = gtk.Button()
        self._open_button = gtk.Button()
        self._refresh_button = gtk.ToolButton(gtk.STOCK_REFRESH)
        self._refresh_button.set_expand(False)
        hbuttonbox.pack_start(self._copy_button, True, True, 0)
        hbuttonbox.pack_start(self._open_button, True, True, 0)
        hbuttonbox.pack_start(self._refresh_button, True, True, 0)
        self._button_setup(self._copy_button, self._open_button,
            self._refresh_button)

        self._treeview = gtk.TreeView()
        scroll.add(self._treeview)
        self._model = gtk.ListStore(str, str, str, str, str, object, object)
        self._treeview.set_model(self._model)
        self._treeview.set_search_column(self.COL_TITLE)
        def search_equal_func(model, column, key, iter):
            if key.upper() in model.get_value(iter, column).upper():
                return False
            return True
        self._treeview.set_search_equal_func(search_equal_func)
        def session_clicked_cb(tv, event, self):
            if not self._event:
                logging.error('Schedule clicked with no event connected')
                return
            logstring = self._event.logstring
            if not logstring:
                logging.debug('Schedule clicked, but no event logging'
                ' specified')
                return
            i = tv.get_path_at_pos(int(event.x), int(event.y))
            if not i:
                return
            row = tv.get_model()[i[0][0]]
            if not row[self.COL_DATETIME]:
                return
            date_time = row[self.COL_DATETIME]
            url = logstring % (date_time.year, date_time.month, date_time.day)
            gtk.show_uri(None, url, gtk.gdk.CURRENT_TIME)
        self._treeview.connect('button-release-event', session_clicked_cb, self)
        self._treeview.set_headers_visible(False)

        column = gtk.TreeViewColumn()
        cell = gtk.CellRendererPixbuf()
        cell.set_property('xalign', 0.5)
        column.pack_start(cell, True)
        column.set_attributes(cell, stock_id=self.COL_ICON)
        column.set_sizing(gtk.TREE_VIEW_COLUMN_FIXED)
        column.set_fixed_width(30)
        self._treeview.append_column(column)

        column = gtk.TreeViewColumn(_('Date'))
        cell = gtk.CellRendererText()
        cell.set_property('xpad', 6)
        column.pack_start(cell, False)
        column.set_attributes(cell, text=self.COL_DATE)
        column.set_alignment(0.5)
        self._treeview.append_column(column)

        column = gtk.TreeViewColumn(_('Starts'))
        cell = gtk.CellRendererText()
        cell.set_property('xpad', 6)
        column.pack_start(cell, False)
        column.set_attributes(cell, text=self.COL_START)
        column.set_alignment(0.5)
        self._treeview.append_column(column)

        column = gtk.TreeViewColumn(_('Ends'))
        cell = gtk.CellRendererText()
        cell.set_property('xpad', 6)
        column.pack_start(cell, False)
        column.set_attributes(cell, text=self.COL_END)
        column.set_alignment(0.5)
        self._treeview.append_column(column)

        column = gtk.TreeViewColumn(_('Title'))
        cell = gtk.CellRendererText()
        cell.set_property('xpad', 6)
        column.pack_start(cell, True)
        column.set_attributes(cell, markup=self.COL_TITLE)
        label = gtk.Label(_('Title'))
        label.set_padding(6, 0)
        label.show()
        column.set_widget(label)
        self._treeview.append_column(column)

        self._schedule = None
        self._current_session = None
        self._update_handle = None
        self._ical = None
        self._icalurl = None
        self._connected = False
        self._ircdate = (dt.now_utc() - timedelta(minutes=70)).date()
        self._cancellable = None
        self._event = None

        self.show_all()

    def _scramble_case(self, string):
        if '@' in string:
            pstring = string.split('@')
            newstring = pstring[0] + '@'
            string = pstring[1]
        else:
            newstring = ''
        random.seed()
        for s in string:
            if random.random() >= .5:
               newstring += s.upper()
            else:
               newstring += s.lower()
        return newstring
        # scramble the case of the url for this event connection.
        # Avoids occasional problem with gvfsd-http wedging 
        # particular url's after loss of connectivity

    def do_event_connect(self, event_man, event):
        self._event = event_man.current_event()
        self._connected = True
        self._last_upcoming_announce = None
        def finished_initial_ical_load_cb(gfile, result, unused):
            ical = read_ical(gfile, result, (event,self._ical))
            if ical == '': 
                return # Do nothing for cancelled I/O Operation
            self._ical = ical
            logging.debug('Finished initial ical load')
            if not (gfile.get_uri() == self._icalurl): # Sanity Check
                logging.debug('Old I/O completion ignored')
                return
            self._cancellable = None
            if not self._connected or not self._event:
                return
            self._update(event, self._ical, initial=True)
            secs = time.localtime().tm_sec
            self._update_handle = glib.timeout_add_seconds(65-secs, self._calendar_refresh, event)
        parsed = [j for j in urlparse.urlparse(event.icalurl)]
        parsed[1]= self._scramble_case(parsed[1])
        self._icalurl = urlparse.urlunparse(parsed)
        cancellable = gio.Cancellable()
        start_read_ical(self._icalurl, finished_initial_ical_load_cb,
            cancellable, (None, None))
        self._cancellable = cancellable
        logging.debug('Started initial ical load %s' % self._icalurl)

    def _calendar_refresh(self, event, force_update=False):
        def finished_refresh_ical_load_cb(gfile, result, data):
            force_reload, irclog_force = data
            ical = read_ical(gfile, result, (event, self._ical))
            if ical == '':
                return # Do nothing for a cancelled calendar read
            if not (gfile.get_uri() == self._icalurl): # Sanity Check
                logging.debug('Old I/O completion ignored')
                return
            self._cancellable = None
            logging.debug('Finished ical load')
            if not self._connected or not self._event:
                return
            if ical == self._ical and not irclog_force:
                logging.debug('Skipping Update--Calendar Unchanged')
                self._update_currency()
            else:
                self._ical = ical
                logging.debug('Updating Schedule')
                self._update(event, self._ical)
            secs = time.localtime().tm_sec
            self._update_handle = glib.timeout_add_seconds(65-secs, self._calendar_refresh, event)
            return

        self._update_handle = None
        irc_force = False
        now = dt.now_utc()
        trigger_time = now.minute in [1, 6, 11, 21, 31, 36, 41, 51]
        trigger_time = trigger_time and not Options.get('no-update')
        # Trigger in time to be done at :50, :00, :20, and :30 so that session
        # announcements will be made on time (bearing in mind low bandwidth
        # connections and the large calendar size
        ircdate = (now - timedelta(minutes=71)).date()
        if ircdate != self._ircdate and trigger_time:
            self._ircdate = ircdate
            irc_force = True
            logging.debug('Daily full calendar update %s' % now)
        if irc_force or force_update or trigger_time:
            parsed = [j for j in urlparse.urlparse(event.icalurl)]
            parsed[1]= self._scramble_case(parsed[1])
            self._icalurl = urlparse.urlunparse(parsed)
            cancellable = gio.Cancellable()
            start_read_ical(self._icalurl, finished_refresh_ical_load_cb, 
                cancellable, (force_update, irc_force))
            self._cancellable = cancellable
            logging.debug('Started ical load')
        else:
            self._update_currency()
            secs = time.localtime().tm_sec
            self._update_handle = glib.timeout_add_seconds(65-secs, self._calendar_refresh, event) 
        return False

    def _update(self, event, ical, initial=False):
        def quote(text):
            quote_table = {
                 '<' : '&lt;',
                 '>' : '&gt;',
                 "'" : '&apos;',
                 '"' : '&quot;',
                 '&' : '&amp;'
                }
            return_value = ''
            for l in text:
                return_value= return_value + quote_table.get(l,l)
            return return_value
        def insert_log_line(date, session):
            difference =  dt.now_utc() - date
            if difference < timedelta(minutes=70):
                return # There may not be a log yet
            sessionrow = ['', '', '', '']
            formatted_date = date.strftime('%x')
            localtime = dt.as_local(date)
            formatted_local = localtime.strftime('%H:00')
            formatted = \
            _('<u>Irc Logs for UTC %s </u><i>(Starting at %s local time)</i>') \
            % (formatted_date, formatted_local)
            sessionrow.append(formatted)
            sessionrow.append(session)
            sessionrow.append(date)
            self._model.append(sessionrow)
        
        def insert_blank_lines(number):
            for i in range(number):
                sessionrow = ['', '', '', '', '', None, None]
                self._model.append(sessionrow)

        self._schedule = parse_ical(event, ical)
        self._model.clear()
        self._scroll_to = None
        self._last_date = None
        current_row = False
        for session in self._schedule:
            utc_starttime = dt.as_utc(session.start_datetime)
            utc_date= dt.datetime(utc_starttime.year, utc_starttime.month,
                utc_starttime.day, tzinfo=utc_starttime.tzinfo)
            if (not utc_date == self._last_date):
                self._last_date = utc_date
                if event.logstring:
                    insert_log_line(utc_date, session)
            sessionrow = []
            sessionrow.append('')
            sessionrow.append(session.start_local_date)
            sessionrow.append(session.start_local_time)
            sessionrow.append(session.end_local_time)
            instructors = reduce(lambda x, y: x + y + ', ', session.instructors, '  ')[:-2]
            title = quote(session.title)
            if session.event:
                sessionrow.append(title + '<small>' + instructors + '</small> <i>' + quote(session.event) +'</i>')
            else:
                sessionrow.append(title + '<small>' + instructors + '</small>')
            sessionrow.append(session)
            sessionrow.append(None)
            current_row = self._model.append(sessionrow)
            if not self._scroll_to and session.state in (session.FUTURE, session.NOW):
                self._scroll_to = self._model.get_path(current_row)
        if not self._scroll_to and current_row:
            self._scroll_to = self._model.get_path(current_row)
        self._update_currency()

        insert_blank_lines(30)

        if initial and self._scroll_to:
            self._treeview.scroll_to_cell(self._scroll_to, use_align=True, row_align=.2)


        self._treeview.set_headers_visible(True)

    def do_event_disconnect(self, event_man, event):
        self._connected = False
        self._schedule = []
        self._current_session = None
        self._model.clear()
        if self._update_handle:
            glib.source_remove(self._update_handle)
            self._update_handle = None
        if self._cancellable:
            self._cancellable.cancel()
            self._cancellable = None
        self._treeview.set_headers_visible(False)
        self._event = None

    def _update_currency(self):
        ended = False
        if self._current_session and self._current_session.state == Session.PAST:
            ended = True
            self._current_session = None
        for i, row in enumerate(self._model):
            session = row[self.COL_SESSION]
            if session == None:
                continue
            if self._current_session and self._current_session.uid == session.uid:
                self._current_session = session  
            if session.state == Session.NOW:
                row[self.COL_ICON] = gtk.STOCK_GO_FORWARD
                if session != self._current_session:
                    if self._current_session == None:
                        session_message = _("Please keep the Session "
                        "tab selected to see session"
                        " learning materials.")
                        self.show_notification(
                            _('Session started'),
                            _('The session "{0}" has started. ').format(session.title) +
                            session_message
                        )
                    self._current_session = session
                    logging.debug('session changed')
                    self.emit('session-changed', session)
            if session.state == Session.FUTURE:
                row[self.COL_ICON] = None
                if timedelta(minutes=9) < session.start_datetime - dt.now_local() < timedelta(minutes=10):
                    if not self._last_upcoming_announce == session.uid:
                        self._last_upcoming_announce = session.uid
                        self.show_notification(
                            _('Session upcoming'),
                            _('The session "{0}" will begin in 10 minutes.').format(session.title)
                    )
            if session.state == Session.PAST:
                row[self.COL_ICON] = gtk.STOCK_NO
        if not self._current_session and ended:
            logging.debug('session ended')
            self.emit('session-ended')
        return True

    def get_current_session(self):
        return self._current_session

    def on_faculty(self,name):
        return self._current_session and name.lower() in \
	[i.lower() for i in self._current_session.instructors + self._current_session.helpers]

    def get_question_token(self):
        if self._current_session:
            if self._current_session.question_token:
                return self._current_session.question_token
            if self._current_session.locale:
                if self._current_session.locale.startswith('es'):
                    return 'PREGUNTA:'
                else:
                    return 'QUESTIION:'
        event = self._event
        if event and event.question_token:
            return event.question_token
        if event and event.locale:
            if event.locale.startswith('es'):
                return 'PREGUNTA:'
            return 'QUESTION:'
        return 'QUESTION:'

    def show_notification(self, summary, body):
        n = pynotify.Notification(summary, body, 'lernid')
        try:
            n.show()
        except:
            logging.debug("Notification failed from Schedule Widget")

    def _button_setup(self, copy_button, open_button, refresh_button):
        copy_button.set_label(_("Copy Calendar ICAL URL to Clipboard"))
        open_button.set_label(_("Open Calendar ICAL URL Using Your Browser"))
        copy_button.connect("clicked", self._copy_button_clicked)
        open_button.connect("clicked", self._open_button_clicked)
        refresh_button.connect("clicked", self._refresh_button_clicked)

    def _copy_button_clicked(self, widget):
        clipboard = gtk.Clipboard()
        event = self._event
        if not event:
            return
        clipboard.set_text(event.icalurl)

    def _open_button_clicked(self, widget):
        event = self._event
        if not event:
            return
        gtk.show_uri(None, event.icalurl, gtk.gdk.CURRENT_TIME)

    def _refresh_button_clicked(self, widget):
        event = self._event
        if not event:
            return
        if self._update_handle:
           glib.source_remove(self._update_handle)
           self._update_handle = None
           self._calendar_refresh(event, force_update=True)



#!/usr/bin/python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
# Copyright (c) 2011 John S Gruber <johnsgruber@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import os
import gtk
import gobject
import poppler
import re
import logging
import gio
import cairo
import StringIO
import glib
import random

from lernid.widgets.Widget import Widget
from lernid.lernidconfig import save_cache_path
from lernid.Statusbar import Statusbar

class Slide(Widget):

    __gtype_name__ = 'LernidSlide'

    __gsignals__ = {
        'slides-downloaded': (
            gobject.SIGNAL_RUN_LAST, None, ()
        ),
        'slide-changed': (
            gobject.SIGNAL_RUN_LAST, None, ()
        ),
    }

    SLIDE_START_SIZE = 300
    SLIDE_MIN_SIZE = 150

    def __init__(self, vertical=False):
        Widget.__init__(self, 'slide')
        self._vertical = vertical
        self._slidebutton_cb = None
        align = gtk.Alignment(0.5, 0.5, 1.0, 1.0)
        if vertical:
            align.set_padding(0, 5, 3, 3)
        else:
            align.set_padding(3, 5, 0, 4)
        self.add(align)
        frame = gtk.Frame()
        frame.set_shadow_type(gtk.SHADOW_IN)
        align.add(frame)
        self._vbox = gtk.VBox()
        frame.add(self._vbox)
        self._top = gtk.EventBox()
        self._scroll = gtk.ScrolledWindow()
        if vertical:
            self._scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_NEVER)
        else:
            self._scroll.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
        self._image = gtk.Image()
        self._scroll.add_with_viewport(self._image)
        self._top.add(self._scroll)
        self._vbox.add(self._top)
        self._eventbox = gtk.EventBox()
        self._slidebutton = gtk.Button()
        self._slidelabel = gtk.Label()
        self._slidelabel.set_line_wrap(True)
        self._slidelabel.set_justify(gtk.JUSTIFY_CENTER)
        self._slidelabel.set_text(_('Click to Open in External Browser'))
        self._slidebutton.add(self._slidelabel)
        self._eventbox.add(self._slidebutton)
        self._eventbox.set_tooltip_text(_('Click to open the slides in ' 
            'your browser. From there you can browse, bookmark, or save them.'))
        self._vbox.pack_start(self._eventbox, expand=False)
        viewport = self._scroll.get_child()
        viewport.set_shadow_type(gtk.SHADOW_NONE)
        self._master_slide = None
        self._session_slide_downloaded = False
        self._cancellable = None
        self._slide_retry = None

        self._last_width = None
        self._last_height = None
        self.connect('expose-event', self._resize)
        self.set_no_show_all(False)
        self.hide()

    def do_event_connect(self, eventman, event):
        classroom = eventman.get_widget_by_name('classroom')
        schedule = eventman.get_widget_by_name('schedule')
        self._on_faculty = schedule.on_faculty # retrieve method
        self.event_connect_signal(classroom, 'message-received', self._classroom_msg_received)
        schedule = eventman.get_widget_by_name('schedule')
        self.event_connect_signal(schedule, 'session-changed', self._session_changed)
        self.event_connect_signal(schedule, 'session-ended', self._session_ended)

    def do_event_disconnect(self, eventman, event):
        self._image.clear()
        self.hide()
        if self._slidebutton_cb:
            self._slidebutton.disconnect(self._slidebutton_cb)
            self._slidebutton_cb = None
        Statusbar.pop_message('slidesession')
        if self._slide_retry:
            glib.source_remove(self._slide_retry)
            self._slide_retry = None
        if self._cancellable:
            self._cancellable.cancel()
            self._cancellable = None
        self.event_disconnect_signals()

    def _classroom_msg_received(self, classroom, chan, sender, text):
        matches = re.search(r'\[(?i)SLIDE\s+(\d+).*\]', text)
        if matches and self._session_slide_downloaded:
            if self._on_faculty(sender):
                self._change_slide_page(int(matches.groups()[0]))
                return
        matches = re.search(r'\[(?i)SLIDEFILE\s+(\S+)\s+(\d+).*\]', text)
        if matches:
            if self._on_faculty(sender):
                self._download_slides(matches.groups()[0], int(matches.groups()[1]))
                return
        matches = re.search(r'\[(?i)SLIDEFILE\s+(\S+).*\]', text)
        if matches:
            if self._on_faculty(sender):
                self._download_slides(matches.groups()[0], 1)
                return

    def _change_slide_page(self, pagenumber):
        logging.debug('changing slide to page %d' % pagenumber)
        try:
            # FIXME
            slide_url = "file://%s/slides.pdf" % os.path.realpath(save_cache_path('lernid'))
            pdf = poppler.document_new_from_file(slide_url, None)
            if pagenumber < 0 or pagenumber > pdf.get_n_pages():
                return
            page = pdf.get_page(int(pagenumber) - 1)
            w, h = page.get_size()
            w, h = (int(w), int(h))
            surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, w, h)
            context = cairo.Context(surface)
            page.render(context)
            # technique from diogodivision's "BlingSwitcher",
            # through http://www.mikedesjardins.net
            temp = StringIO.StringIO()
            surface.write_to_png(temp)
            temp.seek(0)
            loader = gtk.gdk.PixbufLoader()
            loader.write(temp.getvalue())
            loader.close()
            pixbuf = loader.get_pixbuf()
        except:
            Statusbar.push_message(_('An error was encountered while trying to load slide number {0}'.format(pagenumber)), 'slidesession', duration=15)
            logging.debug("Something went wrong when loading slide %s" % pagenumber)
            return
        self._master_slide = pixbuf
        self.show()
        if self._vertical:
            length = self.allocation.height
        else:
            length = self.allocation.width
        if length < self.SLIDE_START_SIZE:
            if self._vertical:
                self.set_size_request(-1, self.SLIDE_START_SIZE)
            else:
                self.set_size_request(self.SLIDE_START_SIZE, -1)
            self.queue_resize()
        self._last_width = None
        self._last_height = None
        self._resize()
        self.emit('slide-changed')

    def _resize(self, widget=None, data=None):
        if self._master_slide is not None:
            mwidth, mheight = self._master_slide.get_width(), self._master_slide.get_height()
            width, height = self._scroll.allocation.width, self._scroll.allocation.height
            if self._vertical:
                height = max(self.SLIDE_MIN_SIZE, height)
                width = int(float(height)/float(mheight)*mwidth)
            else:
                width = max(self.SLIDE_MIN_SIZE, width)
                height = int(float(width)/float(mwidth)*mheight)
            if width == self._last_width and height == self._last_height:
                return True
            self._last_width = width
            self._last_height = height
            newpb = gtk.gdk.Pixbuf.scale_simple(self._master_slide, width, height, gtk.gdk.INTERP_BILINEAR)
            self._image.clear()
            self._image.set_from_pixbuf(newpb)
            self._image.show()

    @staticmethod
    def _retry(n): # retry generator
        for i in range(n):
            yield random.randint(30,120) # random delay until retry
        while 1: yield False

    def _download_slides(self, slideurl, page):
        if self._slide_retry:
            glib.source_remove(self._slide_retry)
            self._slide_retry = None
        if self._cancellable:
            self._cancellable.cancel()
            self._cancellable = None
        self._retries = self._retry(3).next # retry generator obj. next method
        self._download_slides_attempt(slideurl, page)

    def _download_slides_attempt(self, slideurl, page):
        self._slide_retry = None
        Statusbar.pop_message('slidesession')
        path = os.path.join(save_cache_path('lernid'), 'slides.pdf')
        cancellable = gio.Cancellable()
        try:
            logging.debug('downloading slides from %s' % slideurl)
            slidefile = gio.File(slideurl)
            destfile =  gio.File(path)
# It doesn't seem that the gio.FILE_COPY_OVERWRITE is effective for copy_async
            if destfile.query_exists():
                try:
                    destfile.delete(None)
                except: pass
            def reporthook(done_so_far, size):
                Statusbar.pop_message('slidesession')
                if size == 0:
                    return
                msg = _('Downloading session slides (%i%% of %i KB)...') % (100*(float(done_so_far) / float(size)), float(size)/float(1024))
                Statusbar.push_message(msg, 'slidesession', duration=3)
            def download_done(giofile, result, stuff):
                    self, slideurl, page, cancellable = stuff
                    try:
                        giofile.copy_finish(result)
                        self._cancellable = None
                    except gio.Error, information:
                        Statusbar.push_message(_('An error was encountered while downloading slides'), 'slidesession', duration=120)
                        logging.debug('Error when downloading slides from %s;  %s' % (slideurl, information))
                        secs = self._retries()
                        if secs and not cancellable.is_cancelled():
                            logging.error( \
                               'Retrying slide download in %i secs' % secs)
                            self._slide_retry = glib.timeout_add_seconds(secs,
                                self._download_slides_attempt, slideurl,
                                    page) 
                        else:
                            logging.error('Slide downloading failed')
                        return
                    logging.debug('Slide download completed')
                    Statusbar.push_message(_('Slides have been downloaded'), 'slidesession', duration=10)
                    self._session_slide_downloaded = True
                    self.emit('slides-downloaded')
                    self._top.set_tooltip_text(slideurl)
                    if self._slidebutton_cb:
                        self._slidebutton.disconnect(self._slidebutton_cb)
                    self._slidebutton_cb = self._slidebutton.connect("clicked",
                        self._open_slides_in_browser,
                        slideurl)
                    self._change_slide_page(page)
            slidefile.copy_async(destfile, download_done, reporthook, cancellable=cancellable, user_data=(self, slideurl, page, cancellable), flags=gio.FILE_COPY_OVERWRITE)
            self._cancellable = cancellable
        except gio.Error, information:
            Statusbar.push_message(_('An error was encountered while downloading slides'), 'slidesession', duration=120)
            logging.debug('Error when downloading slides from %s;  %s' % (slideurl, information))

    def _session_changed(self, schedule, session):
        if self._slide_retry:
            glib.source_remove(self._slide_retry)
            self._slide_retry = None
        if self._cancellable:
            self._cancellable.cancel()
            self._cancellable = None
        Statusbar.pop_message('slidesession')
        if session.slides:
            self._download_slides(session.slides, 1)
        else:
            self._image.clear()
            self.hide()
            self._session_slides_downloaded = False
            if self._slidebutton_cb:
                self._slidebutton.disconnect(self._slidebutton_cb)
                self._slidebutton_cb = None
            Statusbar.push_message(_('This session does not use slides'), 'slidesession', duration=10)

    def _session_ended(self, schedule):
        if self._slide_retry:
            glib.source_remove(self._slide_retry)
            self._slide_retry = None
        if self._cancellable:
            self._cancellable.cancel()
            self._cancellable = None
        self._image.clear()
        self.hide()
        Statusbar.pop_message('slidesession')
        self._session_slide_downloaded = False
        if self._slidebutton_cb:
            self._slidebutton.disconnect(self._slidebutton_cb)
            self._slidebutton_cb = None

    def _open_slides_in_browser(self, widget, url):
        gtk.show_uri(None, url, gtk.gdk.CURRENT_TIME)



#!/usr/bin/python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
# Copyright (c) 2011 John S Gruber <johnsgruber@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import glib
import os
import gobject
import random
import dbus.mainloop.glib
dbus.mainloop.glib.DBusGMainLoop(set_as_default = True)

import telepathy
from telepathy.interfaces import *
from telepathy.constants import *

import logging
from lernid.LernidOptions import Options
from lernid.lernidconfig import VERSION
from lernid.Statusbar import Statusbar

DBUS_PROPERTIES = dbus.PROPERTIES_IFACE

def ignore(*args): pass


_server_instances = {}

class Server(gobject.GObject):

    __gsignals__ = {
        'connected': (
            gobject.SIGNAL_RUN_LAST, None, ()
        ),
        'disconnected': (
            gobject.SIGNAL_RUN_LAST, None, ()
        ),
        'identified': (
            gobject.SIGNAL_RUN_LAST, None, ()
        ),
    }

    DISCONNECTED, CONNECTING, CONNECTED = range(3)

    def __init__(self, server, nick, eventman):
        gobject.GObject.__init__(self)
        self._eventman = eventman
        self._nick = nick
        self._temp_nick = nick[0:9] + str(random.randint(0,999999))
        self._server = server
        self._channels = {}
        self._private_channels = {}
        self._conn = None
        self._state = self.DISCONNECTED
        self._unconnected_channels = {}
        self._unconnected_private_channels = {}
        self._nickidentifier = None
        self._load_manager()
        self._request_connection(server, self._temp_nick)
        self._conn_timeout = None

    @property
    def nick(self):
        return self._nick

    @nick.setter
    def nick(self, val):
        self._nick = val

    @classmethod
    def get_server(cls, server, nick, eventman):
        logging.debug('getting server')
        global _server_instances
        if server in _server_instances:
            server_obj = _server_instances[server]
            server_obj.nick = nick
            server_obj._request_connection(server, server_obj._temp_nick)
            return server_obj
        else:
            obj = cls(server, nick, eventman)
            _server_instances[server] = obj
            return obj

    def get_channel(self, name):
        if name in self._channels:
            return self._channels[name]
        obj = Channel(name)
        self._channels[name] = obj
        self._connect_channel(obj)
        return obj

    def get_private_channel(self, target):
        if target in self._private_channels:
            return self._private_channels[target]
        obj = PrivateChannel(target)
        self._private_channels[target] = obj
        self._connect_private_channel(obj)
        return obj

    def identify_nick(self, password):
        self._nickidentifier = NickServIdentifier(self.nick, password)
        self._connect_private_channel(self._nickidentifier)
        return self._nickidentifier

    def connection_ready_timeout(self):
        logging.debug('Connection timed out')
        self.disconnect()
        self._eventman.emit('event-interrupted', self._eventman._event)
        return False

    def _load_manager(self):
        logging.debug('loading manager')
        reg = telepathy.client.ManagerRegistry()
        reg.LoadManagers()
        self._cm = reg.GetManager('idle') # IRC

    def _request_connection(self, server, nick):
        logging.debug('requesting connection: %s %s' % (server, nick))
        if self._state == self.DISCONNECTED:
            self._state = self.CONNECTING
            self._cm[CONNECTION_MANAGER].RequestConnection('irc',
                {'account': nick, 'server': server, 'fullname': VERSION},
                reply_handler=self._got_connection,
                error_handler=self.error)
        else:
            logging.debug('ignoring request')

    def _got_connection(self, bus_name, object_path):
        logging.debug('got connection')
        self._conn = telepathy.client.Connection(
            bus_name, object_path,
            ready_handler=self._ready)
        self._conn._acquired_nick = ''
        self._conn[CONNECTION].connect_to_signal('StatusChanged',
                                                 self._status_changed)
        self._conn[CONNECTION].Connect(reply_handler=ignore,
                                       error_handler=self.error)
        if self._conn_timeout:
            glib.source_remove(self._conn_timeout)
        self._conn_timeout = glib.timeout_add_seconds(90,self.connection_ready_timeout)

    def _ready(self, connection):
        logging.debug('server ready')
        self._state = self.CONNECTED
        self._conn[CONNECTION_INTERFACE_RENAMING].RequestRename(self._nick)
        logging.debug('nick potentially changed in _ready handler to %s' % \
            self._nick)
        glib.source_remove(self._conn_timeout)
        self._conn_timeout = None
        for channel in self._unconnected_channels.keys():
            self._connect_channel(channel)
        self._unconnected_channels = {}
        for channel in self._unconnected_private_channels.keys():
            self._connect_private_channel(channel)
        self._unconnected_private_channels = {}

    def _connect_channel(self, channel):
        if self._state != self.CONNECTED:
            self._unconnected_channels[channel] = True
            return
        def setup_channel(yours, object_path, properties):
            channel.do_connect(self._conn, object_path, properties)
        self._conn[CONNECTION_INTERFACE_REQUESTS].EnsureChannel({
                CHANNEL + ".ChannelType": CHANNEL_TYPE_TEXT,
                CHANNEL + ".TargetHandleType": HANDLE_TYPE_ROOM,
                CHANNEL + ".TargetID": channel.name,
            },
            reply_handler = setup_channel,
            error_handler = self.error)

    def _connect_private_channel(self, channel):
        if self._state != self.CONNECTED:
            self._unconnected_private_channels[channel] = True
            return
        def setup_channel(yours, object_path, properties):
            channel.do_connect(self._conn, object_path, properties)
        self._conn[CONNECTION_INTERFACE_REQUESTS].EnsureChannel({
                CHANNEL + ".ChannelType": CHANNEL_TYPE_TEXT,
                CHANNEL + ".TargetHandleType": HANDLE_TYPE_CONTACT,
                CHANNEL + ".TargetID": channel.target,
            },
            reply_handler = setup_channel,
            error_handler = self.error)

    def _status_changed(self, status, reason):
        if status == CONNECTION_STATUS_CONNECTED:
            logging.debug('connected to server')
            self.emit('connected')
        elif status == CONNECTION_STATUS_DISCONNECTED:
            logging.debug('disconnected from server')
            if self._conn_timeout:
                glib.source_remove(self._conn_timeout)
                self._conn_timeout = None
            for chan in self._channels.values():
                chan._c_disconnect()
                self._unconnected_channels[chan] = True
            for chan in self._private_channels.values():
                self._unconnected_private_channels[chan] = True
            if self._nickidentifier:
                self._unconnected_private_channels[self._nickidentifier] = True
            if self._state == self.CONNECTED:
                # Only emit signal if we had a connection before
                self.emit('disconnected')
            self._state = self.DISCONNECTED
            if reason == CONNECTION_STATUS_REASON_NAME_IN_USE:
                self._nick = '{0}{1}'.format(self._nick, random.randint(10, 99))
                self._request_connection(self._server, self._nick)
            if reason == CONNECTION_STATUS_REASON_NETWORK_ERROR:
                self._eventman.emit('event-interrupted', self._eventman._event)

    def disconnect(self):
        if not self._conn:
            return
        def disconnect_cb():
            logging.debug("Server Disconnect Done")
        def disconnect_error_cb(error):
            pass
        if self._conn_timeout:
            glib.source_remove(self._conn_timeout)
            self._conn_timeont = None
        self._conn[CONNECTION].Disconnect(reply_handler=disconnect_cb,
                                          error_handler=disconnect_error_cb)
        self._state = self.DISCONNECTED
        self._conn = None
        self._nickidentifier = None

    def error(self, e):
        logging.debug("ERROR:Server:%s:%s", e.get_dbus_name(), e.args[0])
        self._state = self.DISCONNECTED
        if e.get_dbus_name() == 'org.freedesktop.DBus.Error.ServiceUnknown':
            self._load_manager()
            self._request_connection(self._server, self._temp_nick)
        else:
            self.disconnect()


class Channel(gobject.GObject):

    TYPE_NORMAL = 0
    TYPE_ACTION = 1

    __gsignals__ = {
        'joined': (
            gobject.SIGNAL_RUN_LAST, None, ()
        ),
        'message-received': (
            gobject.SIGNAL_RUN_LAST, None, (str, str, int)
        ),
        'message-sent': (
            gobject.SIGNAL_RUN_LAST, None, (str,)
        ),
        'members-changed': (
            gobject.SIGNAL_RUN_LAST, None, (object,)
        ),
    }

    def __init__(self, name):
        """Channel objects should not be instancted directly, use
        Server.get_channel function."""
        logging.debug('channel created')
        gobject.GObject.__init__(self)
        self._name = name
        self._signal_matches = []
        self._my_handle = None

    def do_connect(self, connection, object_path, properties):
        """Should only be called by Server"""
        logging.debug('connecting channel')
        self._members = {}
        self._conn = connection
        self._chan = telepathy.client.Channel(
            connection.service_name,
            object_path,
            ready_handler=self._ready)

    @property
    def name(self):
        return self._name

    def get_members(self):
        return self._members.values()

    def send_message(self, message):
        if not self._chan:
            return
        def ack():
            logging.debug('message sent to %s' % self.name)
            self.emit('message-sent', message)
        self._chan[CHANNEL_TYPE_TEXT].Send(0, message,
                                           reply_handler=ack,
                                           error_handler=self._conn.error)

    def is_moderated(self):
        try:
            return bool(self._chan[PROPERTIES_INTERFACE].GetProperties([3])[0][1])
        except Exception, e:
            logging.debug('could not acess "moderated" property: {0}'.format(e))
            return False

    def _ready(self, channel):
        """We have joined the channel."""
        logging.debug('connected channel %s' % self.name)
        match = self._chan[CHANNEL_INTERFACE_GROUP].connect_to_signal('MembersChanged',
                                     self._members_changed)
        self._signal_matches.append(match)
        self._chan[CHANNEL_INTERFACE_GROUP].GetMembers(reply_handler=self._update_members,
                                                       error_handler=self._conn.error)
        match = self._chan[CHANNEL_TYPE_TEXT].connect_to_signal('Received', self._received_message)
        self._signal_matches.append(match)
        self.emit('joined')

    def _c_disconnect(self):
        for match in self._signal_matches:
            match.remove()
        self._signal_matches = []

    def _update_members(self, handles):
        def update_cb(ids):
            self._members.update(zip(handles, ids))
            if Options.get('more_verbose'):
                logging.debug('members updated')
            self.emit('members-changed', self.get_members())
            if self._my_handle:
                try:
                    my_alias = self._members[self._my_handle]
                except:
                    return
                if not my_alias == self._conn._acquired_nick:
                    self._conn._acquired_nick = my_alias
                    Statusbar.push_message(_('Nick assigned by server: %s' % my_alias), duration=20)
                    logging.debug('Nick assigned by server: %s' % my_alias)
        def update_my_handle(handle):
            self._my_handle = handle
        # If this first call doesn't return before the Aliases come back
        # we can set our alias name the next time membership in a room changes
        self._my_handle = self._conn[CONNECTION].GetSelfHandle(reply_handler=update_my_handle, error_handler=lambda x: None)
        self._conn[CONNECTION_INTERFACE_ALIASING].RequestAliases(
             handles,
             reply_handler=update_cb,
             error_handler=ignore)

    def _members_changed(self, message, added, removed,
                         local_pending, remote_pending, actor, reason):
        if Options.get('more_verbose'):
            logging.debug('members changed in %s' % self.name)
        for h in removed:
            if h in self._members:
                del self._members[h]
        self._update_members(added)

    def _received_message(self, id, timestamp, sender_handle, msgtype, flags, text):
        if Options.get('more_verbose'):
            logging.debug('message received in %s' % self.name)
        if msgtype == CHANNEL_TEXT_MESSAGE_TYPE_ACTION:
            msgtype = self.TYPE_ACTION
        else:
            msgtype = self.TYPE_NORMAL
        if not sender_handle in self._members:
            sender = self._conn[CONNECTION_INTERFACE_ALIASING].RequestAliases(
             [sender_handle])[0] # Need to backstop other alias mechanisms
            self._members[sender_handle] = sender
        else:
            sender = self._members[sender_handle]
        self.emit('message-received', sender, text, msgtype)


class PrivateChannel(gobject.GObject):

    __gsignals__ = {
        'joined': (
            gobject.SIGNAL_RUN_LAST, None, ()
        ),
        'message-received': (
            gobject.SIGNAL_RUN_LAST, None, (str, str)
        ),
        'message-sent': (
            gobject.SIGNAL_RUN_LAST, None, (str,)
        ),
    }

    @property
    def target(self):
        return self._target

    def __init__(self, target):
        """PrivateChannel objects should not be instanced directly, use
        Server.get_private_channel function."""
        logging.debug('private channel created')
        gobject.GObject.__init__(self)
        self._target = target
        self._conn = None
        self._chan = None

    def do_connect(self, connection, object_path, properties):
        """Should only be called by Server"""
        logging.debug('connecting private channel')
        self._conn = connection
        self._chan = telepathy.client.Channel(
            connection.service_name,
            object_path,
            ready_handler=self._ready)

    def send_message(self, message):
        if not self._chan:
            return False
        def ack():
            logging.debug('message sent to %s' % self.target)
            self.emit('message-sent', message)
        self._chan[CHANNEL_TYPE_TEXT].Send(0, message,
                                           reply_handler=ack,
                                           error_handler=self._conn.error)
        return True

    def _ready(self, channel):
        """We have joined the channel."""
        logging.debug('connected private channel %s' % self.target)
        self._chan[CHANNEL_TYPE_TEXT].connect_to_signal('Received', self._received_message)
        self.emit('joined')

    def _received_message(self, id, timestamp, sender_handle, msgtype, flags, text):
        logging.debug('private message by %s is %s' % (self.target,text))
        self.emit('message-received', self.target, text)


class NickServIdentifier(PrivateChannel):
    """A class for identifying with NickServ.
    Should only be instanciated by Server object
    """

    __gsignals__ = {
        'identified': (
            gobject.SIGNAL_RUN_LAST, None, ()
        ),
        'invalid-password': (
            gobject.SIGNAL_RUN_LAST, None, ()
        ),
    }

    def __init__(self, nick, password):
        PrivateChannel.__init__(self, 'NickServ')
        self._nick = nick
        self.connect('joined', self._joined, password)
        self.connect('message-received', self._msg_received)

    def retry(self, password):
        """Try indentifying with NickServ again.
        To be used if initial identification fails because of invalid password.
        """
        self._identify(password)

    def _joined(self, chan, password):
        self._identify(password)

    def _identify(self, password):
        self.send_message('identify {0} {1}'.format(self._nick, password))

    def _msg_received(self, chan, sender, msg):
        if msg.startswith('You are now identified'):
            logging.debug('identified nick')
            handle = self._conn[CONNECTION].GetSelfHandle()
            logging.debug('our handle before the rename is %s' %  self._conn[CONNECTION_INTERFACE_ALIASING].RequestAliases([handle])[0])
            self._conn[CONNECTION_INTERFACE_RENAMING].RequestRename(self._nick)
            logging.debug('nick potentially changed in ident. handler to %s'% \
                self._nick)
            self.emit('identified')
        elif msg.startswith('Invalid password'):
            logging.debug('invalid password')
            self.emit('invalid-password')



#!/usr/bin/python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
# Copyright (c) 2011 John S Gruber <johnsgruber@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import os
import tempfile
import gtk
import glib
import gobject
import ConfigParser
import urllib2
import io
import logging

from lernid import ConnectDialog
from lernid.PasswordDialog import PasswordDialog
from lernid.lernidconfig import save_cache_path
from lernid.lernidconfig import __lernid_data_directory__
from lernid.LernidPreferences import Preferences
from lernid.Event import Event
from lernid.LernidOptions import Options
from lernid.Statusbar import Statusbar
from lernid.IrcBackend import Server



_instance_object = None

class EventManager(gobject.GObject):

    __gtype_name__ = 'EventManager'

    __gsignals__ = {
        'event-connect': (
            gobject.SIGNAL_RUN_LAST, None, (gobject.TYPE_PYOBJECT,)
        ),
        'event-disconnect': (
            gobject.SIGNAL_RUN_LAST, None, (gobject.TYPE_PYOBJECT,)
        ),
        'event-interrupted': (
            gobject.SIGNAL_RUN_LAST, None, (gobject.TYPE_PYOBJECT,)
        ),
    }

    def __init__(self):
        gobject.GObject.__init__(self)
        self._cachefolder = save_cache_path('lernid')
        self._cachefolder_events = os.path.join(self._cachefolder, 'events')
        self._isconnected = False
        self._connectdialog = None
        self._passworddialog = None
        self._widgets = {}
        self._config = ConfigParser.ConfigParser()
        self._defaultconfig = u"""[#ubuntu-classroom -- In English] 
homepage: https://wiki.ubuntu.com/Classroom
classroom: ubuntu-classroom
chat: ubuntu-classroom-chat
icalurl: http://www.google.com/calendar/ical/canonical.com_qg6t4s8i7mg8d4lgfu9f93qid4@group.calendar.google.com/public/basic.ics

[#ubuntu-charlas -- En Español]
homepage: https://wiki.ubuntu.com/Classroom
classroom: ubuntu-charlas
Chat:ubuntu-charlas-chat
locale: es.ES
icalurl: http://www.google.com/calendar/ical/q16aejpuv5kgmoa7b6ridenkpk%40group.calendar.google.com/public/basic.ics
"""

# Make sure that these names are lower case
        self.classbotnames = ['classbot', 'classroombot', 'clasebot']
        if Options.get('config'):
            try:
                self._configtext = self._read_config()
            except IOError:
                Statusbar.push_message(_('Cannot retrieve configuration'))
                self._configtext = self._defaultconfig
            tf = tempfile.TemporaryFile()
            tf.write(self._configtext)
            tf.seek(0)
            self._config.readfp(tf)
            tf.close()
        else:
            if __lernid_data_directory__.startswith('/'):
                path = '/etc/lernid-classrooms.d'
            else:
                path = os.path.join(os.path.dirname(__file__),'../etc/lernid-classrooms.d')
            path = os.path.abspath(path)
            self._config.read([path+'/'+i for i in os.listdir(path)])
             
    @classmethod
    def get_instance(cls):
        global _instance_object
        if not _instance_object:
            _instance_object = cls()
        return _instance_object

    def _read_config(self):
        uri = Options.get('config')
        if uri.startswith('http://') or uri.startswith('https://'):
            return urllib2.urlopen(uri,None,30).read()
        else:
            try:
                return open(uri).read()
            except IOError:
                logging.critical("Unable to read configuration file "+uri)
                raise

    def _connect_event(self, event, nick):
        if self.is_connected():
            self.disconnect()
        if not self._config.has_option(event, 'eventstart'):
            eventstart = '2011-01-01 00:00:00'
        else:
            eventstart   = self._config.get(event, 'eventstart')
        if not self._config.has_option(event, 'eventend'):
            eventend = '2029-01-01 23:59:59'
        else:
            eventend   = self._config.get(event, 'eventend')
        try:
            homepage = self._config.get(event, 'homepage')
        except:
            homepage = "https://wiki.ubuntu.com/Classroom"
        try:
            icalurl = self._config.get(event, 'icalurl')
        except:
            icalurl = "http://www.google.com/calendar/ical/canonical.com_qg6t4s8i7mg8d4lgfu9f93qid4@group.calendar.google.com/public/basic.ics"
        if Options.get('calendar'):
            icalurl = Options.get('calendar')
        try:
            classroom = self._config.get(event, 'classroom')
            if classroom[0] not in '#&!+':
                classroom = '#' + classroom
        except:
            classroom = ""
        try:
            chat = self._config.get(event, 'chat')
            if chat[0] not in '#&!+':
                chat = '#' + chat
        except:
            chat = "#ubuntu-classroom-chat"
        try:
            logstring= self._config.get(event, 'logstring')
        except:
            logstring = None
        try:
            locale = self._config.get(event, 'locale')
        except:
            locale = None
        try:
            server = self._config.get(event, 'server')
        except:
            server = None
        try:
            question_token = self._config.get(event, 'question_token')
        except:
            question_token = None

        self._event = Event(
            name = event,
            nick = nick,
            homepage = homepage,
            icalurl = icalurl,
            classroom = classroom,
            chat = chat,
            eventstart=eventstart,
            eventend=eventend,
            locale = locale,
            question_token = question_token,
            logstring = logstring,
            server = server
        )
        logging.debug('connecting to event {0}'.format(event))
        Statusbar.push_message(_('Connecting to event'), duration=10)
        self._isconnected = True
        self.emit('event-connect', self._event)

    def _identify_nick(self, nick, password):
        server = Server.get_server('irc.freenode.net', nick, self)
        identifier = server.identify_nick(password)
        def update_statusbar(obj):
            Statusbar.push_message(_('Your nickname is now identified'), duration=15)
        identifier.connect('identified', update_statusbar)
        def retry(obj):
            if not self._passworddialog:
                self._passworddialog = PasswordDialog.new()
            password = self._passworddialog.run()
            if password != None:
                identifier.retry(password)
        identifier.connect('invalid-password', retry)


    def connect_dialog(self, widget=None, parent=None):
        """Show the connect dialog and connect to the resources."""
        if not self._connectdialog:
            self._connectdialog = ConnectDialog.NewConnectDialog()
            self._connectdialog.set_transient_for(parent)
            self._connectdialog.connect('response', self._connect_dialog_first_response)
            config_event = Preferences.get('event')
            if config_event:
                self._connectdialog.set_event(config_event[1:-1])
            self._connectdialog.set_nick(Preferences.get('nick'))
            self._connectdialog.set_transient_for(widget)
        self._connectdialog.show()

    def _connect_dialog_first_response(self, dialog, response):
        if response == gtk.RESPONSE_OK:
            self._connectdialog.hide()
            glib.idle_add(self._connect_dialog_response, dialog, response)

    def _connect_dialog_response(self, dialog, response):
        event = self._connectdialog.get_event()
        Preferences.set('event', '"%s"' % event)
        nick = self._connectdialog.get_nick()
        Preferences.set('nick', nick)
        self._connect_event(event, nick)
        password = self._connectdialog.get_password()
        if password:
            self._identify_nick(nick, password)

    def disconnect(self):
        if not self.is_connected():
            return
        self._isconnected = False
        Statusbar.push_message(_('Disconnecting from event'), duration=10)
        self.emit('event-disconnect', self._event)
        logging.debug('disconnecting')

    def current_event(self):
        return self._event

    def is_connected(self):
        return self._isconnected

    def get_available_events(self):
        return self._config.sections()

    def register_widget(self, widget):
        if widget.name in self._widgets:
            raise ValueError('a widget with that name is already registered')
        self._widgets[widget.name] = widget

    def get_widget_by_name(self, name):
        if name in self._widgets:
            return self._widgets[name]
        return None




# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gtk

from lernid.lernidconfig import get_data_path

class PasswordDialog(gtk.Dialog, gtk.Buildable):
    __gtype_name__ = "PasswordDialog"

    @staticmethod
    def new():
        builder = gtk.Builder()
        builder.add_from_file(get_data_path('ui', 'PasswordDialog.ui'))    
        return builder.get_object("password_dialog")

    def do_parser_finished(self, builder):
        """do_parser_finished is automatically called after
        the UI definition has been parsed.
        """
        entry = builder.get_object('password_entry')
        ok = builder.get_object('ok_button')
        def update(widget):
            ok.set_sensitive(bool(entry.get_text()))
        entry.connect('changed', update)

        self.entry = entry

    def run(self):
        password = None
        if gtk.Dialog.run(self) == gtk.RESPONSE_OK:
            password = self.entry.get_text()
        self.entry.set_text('')
        self.hide()
        return password


if __name__ == "__main__":
    dialog = PasswordDialog.new()
    print dialog.run()



# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import os
import gtk

from lernid.lernidconfig import *


class AboutLernidDialog(gtk.AboutDialog):
    __gtype_name__ = "AboutLernidDialog"

    def __init__(self):
        """__init__ - This function is typically not called directly.
        Creation of a AboutLernidDialog requires redeading the associated ui
        file and parsing the ui definition extrenally,
        and then calling AboutLernidDialog.finish_initializing().

        Use the convenience function NewAboutLernidDialog to create
        NewAboutLernidDialog objects.

        """
        gtk.AboutDialog.__init__(self)

    def finish_initializing(self, builder):
        """finish_initalizing should be called after parsing the ui definition
        and creating a AboutLernidDialog object with it in order to finish
        initializing the start of the new AboutLernidDialog instance.

        """
        #get a reference to the builder and set up the signals
        self.builder = builder
        self.builder.connect_signals(self)

        self.set_comments(_(DESCRIPTION))
        self.set_version(VERSION)
        self.set_website(WEBSITE)
        self.set_authors(CONTRIBUTORS)
        self.set_artists(ARTISTS)
        self.set_license(LICENSE)

        #code for other initialization actions should be added here

def NewAboutLernidDialog():
    """NewAboutLernidDialog - returns a fully instantiated
    AboutLernidDialog object. Use this function rather than
    creating a AboutLernidDialog instance directly.

    """

    #look for the ui file that describes the ui
    ui_filename = get_data_path('ui', 'AboutLernidDialog.ui')
    builder = gtk.Builder()
    builder.add_from_file(ui_filename)
    dialog = builder.get_object("about_lernid_dialog")
    dialog.finish_initializing(builder)
    return dialog

if __name__ == "__main__":
    dialog = NewAboutLernidDialog()
    dialog.show()
    gtk.main()




#!/usr/bin/python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import time
from datetime import datetime, timedelta, tzinfo

ZERO = timedelta(0)

class UTCTimezone(tzinfo):
    """UTC"""

    def utcoffset(self, dt):
        return ZERO

    def tzname(self, dt):
        return "UTC"

    def dst(self, dt):
        return ZERO

UTC = UTCTimezone()

STDOFFSET = timedelta(seconds = -time.timezone)
if time.daylight:
    DSTOFFSET = timedelta(seconds = -time.altzone)
else:
    DSTOFFSET = STDOFFSET
DSTDIFF = DSTOFFSET - STDOFFSET

class LocalTimezone(tzinfo):
    """Local timezone from http://docs.python.org/library/datetime.html#datetime.datetime"""

    def utcoffset(self, dt):
        if self._isdst(dt):
            return DSTOFFSET
        else:
            return STDOFFSET

    def dst(self, dt):
        if self._isdst(dt):
            return DSTDIFF
        else:
            return ZERO

    def tzname(self, dt):
        return time.tzname[self._isdst(dt)]

    def _isdst(self, dt):
        tt = (dt.year, dt.month, dt.day,
              dt.hour, dt.minute, dt.second,
              dt.weekday(), 0, -1)
        stamp = time.mktime(tt)
        tt = time.localtime(stamp)
        return tt.tm_isdst > 0

Local = LocalTimezone()


def now_local():
    return datetime.now(Local)

def now_utc():
    return datetime.now(UTC)

def parse_time(source, format, source_tz=UTC, target_tz=Local):
    return datetime(*(time.strptime(source, format)[0:6]), tzinfo=source_tz).astimezone(target_tz)

def as_local(dt):
    return dt.astimezone(Local)

def as_utc(dt):
    return dt.astimezone(UTC)



#!/usr/bin/python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
# Copyright (c) 2011 John S Gruber <johnsgruber@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import re
import vobject
import urllib2
import logging

import lernid.DateTime as dt
import io
import gio, glib
from datetime import timedelta

half_year = timedelta(days = 183)

class Session(object):

    PAST, NOW, FUTURE = range(3)

    def __init__(self, **kwargs):
        for k, v in kwargs.iteritems():
            if k in ('uid', 'title', 'description', 'instructors', 
               'helpers', 'local_start', 'local_end', 'slides', 
               'event', 'locale', 
               'question_token'):
                    setattr(self, '_'+k, v)

    @property
    def uid(self):
        return self._uid

    @property
    def title(self):
        return self._title

    @property
    def description(self):
        return self._description

    @property
    def instructors(self):
        return getattr(self, '_instructors', [])

    @property
    def helpers(self):
        return getattr(self, '_helpers', [])

    @property
    def locale(self):
        return getattr(self, '_locale', None)

    @property
    def question_token(self):
        return getattr(self, '_question_token', None)

    @property
    def start_local_date(self):
        # Translators: Local date representation
        return self._local_start.strftime(_('%d %B %Y'))

    @property
    def start_local_time(self):
        # Translators: Local time representation
        return self._local_start.strftime(_('%H:%M'))

    @property
    def end_local_time(self):
        # Translators: Local time representation
        return self._local_end.strftime(_('%H:%M'))

    @property
    def start_datetime(self):
        return self._local_start

    @property
    def end_datetime(self):
        return self._local_end

    @property
    def slides(self):
        return getattr(self, '_slides', None)

    @property
    def event(self):
        return getattr(self, '_event', None)

    @property
    def state(self):
        now, start, end = dt.now_local(), self.start_datetime, self.end_datetime
        if now < start:
            return self.FUTURE
        if start < now < end:
            return self.NOW
        if now > end:
            return self.PAST

def start_read_ical(url, when_done_cb, cancellable, data):
    gfile = gio.File(uri=url)
    r = gfile.load_contents_async(when_done_cb, cancellable=cancellable,
        user_data=data)

def read_ical(gfile,result,userdata):
    event, old_ical = userdata
    default_cal_error = (
u"""BEGIN:VCALENDAR
BEGIN:VEVENT
DTSTART:20100101T000000Z
DTEND:20100101T000000Z
DESCRIPTION:CalendarError
SUMMARY: """ +   _('Unable to load calendar %s\n') + 
u"""END:VEVENT
END:VCALENDAR""")   % event.icalurl

    try:
        
        ical = unicode(gfile.load_contents_finish(result)[0])
        icf = io.StringIO(ical)
        ical_output = io.StringIO()
        # DTSTAMP is changed for each calendar entry each time, but isn't
        # significant--remove them
        for i in icf.readlines():
            if i[0:8] != u'DTSTAMP:':
                ical_output.write(i)
        ical = ical_output.getvalue()
        icf.close()
        ical_output.close()
    except gio.Error, information:
        if information.code == gio.ERROR_CANCELLED:
            logging.debug('Ignoring the completion of a cancelled cal. load')
            return ''
        logging.error('Unable to open calendar %s' % gfile.get_uri())
        logging.error(information)
        if old_ical:
            ical = old_ical
        else:
            ical = default_cal_error
    except glib.GError, information:
        logging.error('Unable to open calendar %s' % gfile.get_uri())
        logging.error(information)
        if old_ical:
            ical = old_ical
        else:
            ical = default_cal_error
    return ical


def parse_ical(event, ical):
    default_cal_error = (
u"""BEGIN:VCALENDAR
BEGIN:VEVENT
DTSTART:20100101T000000Z
DTEND:20100101T000000Z
DESCRIPTION:CalendarError
SUMMARY: """ + _('Unable to parse calendar %s\n') + 
u"""END:VEVENT
END:VCALENDAR""") % event.icalurl

    try:
        cal = vobject.readOne(io.StringIO(ical))
    except:
        logging.critical('Error parsing calendar at %s' % event.icalurl)
        cal = vobject.readOne(io.StringIO(default_cal_error))

    sessions = []

    eventstart_local = dt.parse_time(event.eventstart, '%Y-%m-%d %H:%M:%S')
    eventend_local = dt.parse_time(event.eventend, '%Y-%m-%d %H:%M:%S')

    for session in cal.vevent_list:
        session_data = {}
        if not hasattr(session, "dtstart") or not hasattr(session, "dtend"):
            logging.debug("Missing or invalid time for event")
            local_start = eventend_local
            local_end = eventend_local
        else:
            local_start = dt.as_local(session.dtstart.value)
            local_end = dt.as_local(session.dtend.value)
            if hasattr(session, "description"):
                if session.description.value == "CalendarError":
                    local_start = eventend_local
                    local_end = eventend_local
                session_data = parse_ical_description(session.description.value)
        if hasattr(session, "summary"):
            summary = session.summary.value
        else:
            summary = _('Missing Session Name')
        if hasattr(session, "uid"):
            uid = session.uid
        else:
            uid = 0
        if eventstart_local <= local_start <= eventend_local:
            sessions.append(Session(
                title = summary,
                local_start = local_start,
                local_end = local_end,
                uid = uid,
                **session_data))

    # reverse the list to get the events in chronological order
    sessions.sort(lambda x,y: cmp(x.start_datetime, y.start_datetime))
    return sessions

def parse_ical_description(desc):
    session = {}
    for line in desc.splitlines():
        items = [s.strip() for s in line.split(None, 1)]
        if len(items) != 2:
            continue
        key, value = items
        key = key[:-1].lower()
        if key in ('description', 'slides', 'event', 'locale', 'question_token'):
            session[str(key)] = value
        elif key == 'instructor' or key == 'instructors':
            instructors = [s.strip() for s in value.split(',')]
            session['instructors'] = instructors
        elif key == 'helper' or key == 'helpers':
            helpers = [s.strip() for s in value.split(',')]
            session['helpers'] = helpers
    return session



#!/usr/bin/python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE


class Event(object):

    def __init__(self, **kwargs):
        for k, v in kwargs.iteritems():
            if k in ('name', 'nick', 'homepage', 'icalurl', 'classroom',
                     'chat', 'eventstart', 'eventend', 'locale', 'logstring',
                     'question_token', 'server'):
                setattr(self, '_'+k, v)

    @property
    def name(self):
        return self._name

    @property
    def nick(self):
        return self._nick

    @property
    def homepage(self):
        return self._homepage

    @property
    def icalurl(self):
        if self._icalurl.upper().startswith("HTTPS"):
            return self._icalurl[0:4] + self._icalurl[5:]
        else:
            return self._icalurl

    @property
    def classroom(self):
        return self._classroom

    @property
    def chat(self):
        return self._chat

    @property
    def locale(self):
        return self._locale

    @property
    def logstring(self):
        return self._logstring

    @property
    def server(self):
        return self._server

    @property
    def question_token(self):
        return self._question_token

    @property
    def eventstart(self):
        return self._eventstart

    @property
    def eventend(self):
        return self._eventend



#!/usr/bin/python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gtk
import gobject
import pango
import time
import re

from lernid.widgets.Widget import Widget
from lernid.LernidPreferences import Preferences

from lernid import IrcBackend

class IrcWidget(Widget):

    def get_hand_cursor(self):
        if not hasattr(self, "_hand_cursor"):
            self._hand_cursor = gtk.gdk.Cursor(gtk.gdk.HAND2)
        return self._hand_cursor

    def get_regular_cursor(self):
        if not hasattr(self, "_regular_cursor"):
            self._regular_cursor = gtk.gdk.Cursor(gtk.gdk.XTERM)
        return self._regular_cursor

    def do_event_connect(self, event_man, event):
        self._browser = event_man.get_widget_by_name('browser')
        self.get_toplevel().connect('focus-out-event', self._focus_out)
        schedule = event_man.get_widget_by_name('schedule')
        self._on_faculty = schedule.on_faculty # retrieve method
        self._classbotnames = event_man.classbotnames


    def _init_hyperlinks(self):
        hyperlink_tag = self._buffer.create_tag('hyperlink', foreground='#0000ff',
                                                underline=pango.UNDERLINE_SINGLE)
        self._buffer.connect('mark-set', self._mark_set, hyperlink_tag)
        self._textview.connect('motion-notify-event', self._motion_notify, hyperlink_tag)
        self._textview.connect('expose-event', self._draw_red_line)

        self._red_line_mark = self._buffer.create_mark(None, self._buffer.get_end_iter(), True)
        self._visited_since_last_update = False

    def _append_to_buffer(self, text, sender=None, msgtype=None):
        iend = self._buffer.get_end_iter()
        if self._visited_since_last_update and not self.get_toplevel().is_active():
            self._buffer.move_mark(self._red_line_mark, iend)
            self._textview.queue_draw()
            self._visited_since_last_update = False
        if Preferences.get('show_irc_time'):
            self._buffer.insert(iend, '\n[{0}] '.format(time.strftime(_('%H:%M'))))
        else:
            self._buffer.insert(iend, '\n')
        if msgtype == IrcBackend.Channel.TYPE_ACTION:
            self._buffer.insert(iend, ' * ')
            if sender:
                self._buffer.insert_with_tags_by_name(iend, sender+' ', 'gray')
            start_pos = iend.get_offset()
            self._buffer.insert(iend, text)
        else:
            if sender:
                self._buffer.insert_with_tags_by_name(iend, '<%s> ' % sender, 'gray')
                start_pos = iend.get_offset()
                my_nick = self._chan._conn._acquired_nick
                if text.startswith(my_nick) and sender.lower() in self._classbotnames:
                    self._buffer.insert_with_tags_by_name(iend, text, 'highlight','italicize')
                elif my_nick in text and sender != my_nick:
                    self._buffer.insert_with_tags_by_name(iend, text, 'highlight')
                elif self._on_faculty(sender):
                    self._buffer.insert_with_tags_by_name(iend, text, 'highlight')
                elif sender.lower() in self._classbotnames:
                    self._buffer.insert_with_tags_by_name(iend, text, 'italicize')
                else: self._buffer.insert(iend, text)
            else:
                self._buffer.insert(iend, text)

        if type(text)!=unicode:
            text=unicode(text, 'utf-8')
        for url in re.finditer(r"https?://\S+", text):
            self._buffer.apply_tag_by_name('hyperlink',
                self._buffer.get_iter_at_offset(start_pos + url.start()),
                self._buffer.get_iter_at_offset(start_pos + url.end()))

        self._at_bottom = (self._adjust.value == self._adjust.upper -
                           self._adjust.page_size)

    def _mark_set(self, buf, text_iter, mark, tag):
        if mark.get_name() == 'insert' and text_iter.has_tag(tag) and len(buf.get_selection_bounds()) == 0:
            text_iter.backward_to_tag_toggle(tag)
            istart = text_iter.copy()
            text_iter.forward_to_tag_toggle(tag)
            self._browser.set_location(buf.get_text(istart, text_iter))

    def _motion_notify(self, text_view, event, tag):
        x, y = text_view.window_to_buffer_coords(gtk.TEXT_WINDOW_WIDGET,
            int(event.x), int(event.y))
        iter = text_view.get_iter_at_location(x, y)
        if (iter.has_tag(tag)):
            text_view.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(self.get_hand_cursor())
        else:
            text_view.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(self.get_regular_cursor())
        return False

    def _focus_out(self, widget, event):
        self._visited_since_last_update = True
        return False

    def _draw_red_line(self, widget, event):
        if self._buffer.get_char_count() > 0:
            iter = self._buffer.get_iter_at_mark(self._red_line_mark)
            if (iter.get_offset() > 0):
                top, h = self._textview.get_line_yrange(iter)
                w_x, w_y = self._textview.buffer_to_window_coords(gtk.TEXT_WINDOW_TEXT, 0, top + h)
                win = self._textview.get_window(gtk.TEXT_WINDOW_TEXT)
                gc = gtk.gdk.GC(win)
                gc.set_rgb_fg_color(gtk.gdk.Color('darkred'))
                win.draw_line(gc, 0, w_y, (win.get_size())[0], w_y)



# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import os
import gtk
import ConfigParser
import gobject
import urllib
import xdg.BaseDirectory
import logging

from lernid.lernidconfig import get_data_path, save_cache_path

class ConnectDialog(gtk.Dialog):
    __gtype_name__ = "ConnectDialog"

    def __init__(self):
        """__init__ - This function is typically not called directly.
        Creation of a ConnectDialog requires redeading the associated ui
        file and parsing the ui definition extrenally,
        and then calling ConnectDialog.finish_initializing().

        Use the convenience function NewConnectDialog to create
        a ConnectDialog object.

        """
        gtk.Dialog.__init__(self)

    def finish_initializing(self, builder):
        """finish_initalizing should be called after parsing the ui definition
        and creating a ConnectDialog object with it in order to finish
        initializing the start of the new ConnectDialog instance.

        """
        # FIXME: avoid circular import
        from lernid.EventManager import EventManager

        #get a reference to the builder and set up the signals
        self.builder = builder
        self.builder.connect_signals(self)

        store = gtk.ListStore(gobject.TYPE_STRING)
        eventman = EventManager.get_instance()
        self._eventlist=[]
        for s in eventman.get_available_events():
            store.append([s])
            self._eventlist += [unicode(s),]

        self.event_combo = self.builder.get_object("event_combo")
        self.event_combo.set_model(store)
        cell = gtk.CellRendererText()
        self.event_combo.pack_start(cell, True)
        self.event_combo.add_attribute(cell, 'text',0)
        if len(store) > 0:
            self.event_combo.set_active(0)

    def ok(self, widget, data=None):
        """ok - The user has elected to save the changes.
        Called before the dialog returns gtk.RESONSE_OK from run().
        """
        self.hide()
        self.event_combo.grab_focus()

    def cancel(self, widget, data=None):
        """cancel - The user has elected cancel changes.
        Called before the dialog returns gtk.RESPONSE_CANCEL for run()
        """
        self.hide()
        self.event_combo.grab_focus()

    def check_ready_to_connect(self, widget):
        event_combo = self.builder.get_object("event_combo")
        nick_textentry = self.builder.get_object("nick_textentry")
        if self.is_valid_nick(nick_textentry) and (event_combo.get_active() != -1) :
            self.builder.get_object("button_connect").set_sensitive(True)
        else:
            self.builder.get_object("button_connect").set_sensitive(False)

    def get_nick(self):
        return self.builder.get_object('nick_textentry').get_text()

    def set_nick(self, nick):
        return self.builder.get_object('nick_textentry').set_text(nick)

    def get_event(self):
        return self.builder.get_object('event_combo').get_active_text()

    def set_event(self, event):
        if not event in self._eventlist:
            return False
        index = self._eventlist.index(event)
        return self.builder.get_object('event_combo').set_active(index)

    def get_password(self):
        return self.builder.get_object('password').get_text()

    def is_valid_nick(self, nick_te):
        alpha = 'abcdefghijklmnopqrstuvwxyz'
        alpha = alpha + alpha.upper()
        numeric = '0123456789'
        symbol = '_-[]\\`^{}|'
        nick = nick_te.get_text()
        if nick == "":
            nick_te.set_tooltip_text(_("Nick can not be blank"))
            return False
        try:
            if nick[0] not in alpha + symbol:
                nick_te.set_tooltip_text(_("Nick can not begin with: %s" % (nick[0])))
                return False
            for c in nick[1:]:
                if c not in alpha + numeric + symbol:
                    nick_te.set_tooltip_text(_("Nick can not contain blank spaces.") if c == ' ' else _("Nick can not contain: %s" % (c)))
                    return False
        except UnicodeDecodeError:
            nick_te.set_tooltip_text(_("Nick can only contain English alphabet characters"))
            return False
        nick_te.set_tooltip_text(None)
        return True

def NewConnectDialog():
    """NewConnectDialog - returns a fully instantiated
    dialog-camel_case_nameDialog object. Use this function rather than
    creating ConnectDialog instance directly.

    """

    #look for the ui file that describes the ui
    ui_filename = get_data_path('ui', 'ConnectDialog.ui')
    builder = gtk.Builder()
    builder.add_from_file(ui_filename)
    dialog = builder.get_object("connect_dialog")
    dialog.finish_initializing(builder)
    return dialog

if __name__ == "__main__":
    dialog = NewConnectDialog()
    dialog.show()
    gtk.main()





#!/usr/bin/python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gtk
try:
    import vte
    VTE_SUPPORT = True
except ImportError:
    VTE_SUPPORT = False

from lernid.widgets.Widget import Widget

class Terminal(Widget):

    __gtype_name__ = 'LernidTerminal'

    def __init__(self):
        Widget.__init__(self, 'terminal')
        scroll = gtk.ScrolledWindow()
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self._terminal = vte.Terminal()
        self._terminal.fork_command()
        def refork(terminal):
            terminal.fork_command()
        self._terminal.connect('child-exited', refork)
        def paste(terminal, event):
            if not (event.state & gtk.gdk.CONTROL_MASK):
                return
            if event.keyval == ord('V'):
                self._terminal.paste_clipboard()
            return True
        self._terminal.add_events(gtk.gdk.KEY_RELEASE_MASK)
        self.connect('key-release-event', paste)
        scroll.add(self._terminal)
        self.add(scroll)



# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import os
import gtk

from lernid.lernidconfig import get_data_path
from lernid.LernidPreferences import Preferences

class PreferencesLernidDialog(gtk.Dialog):
    __gtype_name__ = "PreferencesLernidDialog"

    def __init__(self):
        """__init__ - This function is typically not called directly.
        Creation of a PreferencesLernidDialog requires redeading the associated ui
        file and parsing the ui definition extrenally,
        and then calling PreferencesLernidDialog.finish_initializing().

        Use the convenience function NewPreferencesLernidDialog to create
        NewAboutLernidDialog objects.
        """
        gtk.Dialog.__init__(self)

    def finish_initializing(self, builder):
        """finish_initalizing should be called after parsing the ui definition
        and creating a AboutLernidDialog object with it in order to finish
        initializing the start of the new AboutLernidDialog instance.
        """

        self._prefs = Preferences.get()

        #get a reference to the builder and set up the signals
        self.builder = builder
        self.show_appindicator = builder.get_object('show_appindicator')
        self.use_vertical_layout = builder.get_object('vertical_layout')
        self.show_irc_time = builder.get_object('show_irc_time')
        self.open_links_in_default_browser = builder.get_object('links_in_default_browser')

        self._load_preferences()
        self.builder.connect_signals(self)

    def _load_preferences(self):
        self.show_appindicator.set_active(self._prefs.get('show_appindicator'))
        self.use_vertical_layout.set_active(self._prefs.get('vertical'))
        self.show_irc_time.set_active(self._prefs.get('show_irc_time'))
        self.open_links_in_default_browser.set_active(self._prefs.get('links_in_default_browser'))

    def ok(self, widget, data=None):
        """ok - The user has elected to save the changes.
        Called before the dialog returns gtk.RESONSE_OK from run().
        """

        self._prefs['show_appindicator'] = self.show_appindicator.get_active()
        self._prefs['vertical'] = self.use_vertical_layout.get_active()
        self._prefs['show_irc_time'] = self.show_irc_time.get_active()
        self._prefs['links_in_default_browser'] = self.open_links_in_default_browser.get_active()

        Preferences.set(self._prefs)

    def cancel(self, widget, data=None):
        """cancel - The user has elected cancel changes.
        Called before the dialog returns gtk.RESPONSE_CANCEL for run()
        """

        #restore any changes to self.__preferences here
        pass

    def restart_required(self, widget):
        self.builder.get_object('restart_required_message').show()

def NewPreferencesLernidDialog():
    """NewPreferencesLernidDialog - returns a fully instantiated
    PreferencesLernidDialog object. Use this function rather than
    creating a PreferencesLernidDialog instance directly.
    """

    #look for the ui file that describes the ui
    ui_filename = get_data_path('ui', 'PreferencesLernidDialog.ui')
    builder = gtk.Builder()
    builder.add_from_file(ui_filename)
    dialog = builder.get_object("preferences_lernid_dialog")
    dialog.finish_initializing(builder)
    return dialog

if __name__ == "__main__":
    dialog = NewPreferencesLernidDialog()
    dialog.show()
    gtk.main()




#!/usr/bin/python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import os.path
import gtk

from lernid.lernidconfig import get_data_path
from lernid.EventManager import EventManager

class Widget(gtk.Bin):

    __gtype_name__ = 'LernidWidget'

    def __init__(self, name):
        gtk.Bin.__init__(self)
        self._name = name
        self._connected_signals = []
        event_man = EventManager.get_instance()
        event_man.register_widget(self)
        event_man.connect('event-connect', self.do_event_connect)
        event_man.connect('event-disconnect', self.do_event_disconnect)

    @property
    def name(self):
        return self._name

    def builder_with_file(self, filename):
        path = get_data_path('ui', filename)
        builder = gtk.Builder()
        builder.add_from_file(path)
        return builder

    def do_size_request(self, req):
        child = self.get_child()
        if child:
            child.size_request()

    def do_size_allocate(self, alloc):
        child = self.get_child()
        if child:
            child.size_allocate(alloc)

    def event_connect_signal(self, obj, signal, callback):
        hid = obj.connect(signal, callback)
        self._connected_signals.append((obj, hid))

    def event_disconnect_signals(self):
        for obj, hid in self._connected_signals:
            if obj.handler_is_connected(hid):
                obj.disconnect(hid)
        self._connected_signals = []

    def do_event_connect(self, eventman, event):
        pass

    def do_event_disconnect(self, eventman, event):
        pass



#!/usr/bin/python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
# Copyright (c) 2011 John S Gruber <johnsgruber@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gtk
import gobject
import urlparse
import logging
import pango
import time

from lernid.LernidPreferences import Preferences
from lernid.widgets.IrcWidget import IrcWidget

from lernid import IrcBackend
from lernid.LernidOptions import Options

class Chatroom(IrcWidget):

    __gtype_name__ = 'LernidNativeChatroom'

    __gsignals__ = {
        'members-changed': (
            gobject.SIGNAL_RUN_LAST, None, (object,)
        ),
    }

    def __init__(self):
        IrcWidget.__init__(self, 'chatroom')
        self.set_property('can-focus', True)

        builder = self.builder_with_file('ChatWidget.ui')

        scroll = builder.get_object('scroll')
        self._adjust = scroll.get_vadjustment()

        self.add(builder.get_object('vbox1'))
        self._textview = builder.get_object('textview')
        self._textview.modify_font(pango.FontDescription('Monospace'))
        self._textview.set_sensitive(False)
        self._textview.connect('size-allocate', self._scroll)
        self._at_bottom = None

        self._buffer = self._textview.get_buffer()
        text_color = self._textview.get_style().text[gtk.STATE_INSENSITIVE]
        self._buffer.create_tag('gray', foreground_gdk=text_color)
        self._buffer.create_tag('highlight', weight=700)
        self._buffer.create_tag('italicize', style=pango.STYLE_ITALIC)
        self._bold = pango.FontDescription()
        self._bold.set_weight(700)

        self._input = builder.get_object('input')
        self._input.connect('icon-press', self._send_text)
        self._input.connect('activate', self._send_text)
        self._input.connect('key-press-event', self._key_pressed)
        self._input.modify_font(pango.FontDescription('Monospace'))

        self._questioncheckbox = builder.get_object('questioncheckbox')

        self._init_hyperlinks()

    def do_event_connect(self, eventman, event):
        IrcWidget.do_event_connect(self, eventman, event)
        self._server = IrcBackend.Server.get_server('irc.freenode.net', event.nick, eventman)
        self._nick = event.nick
        channelname = Options.get('chatroom', event.chat)
        if len(channelname) == 0:
            return
        if channelname[0] not in '#&!+':
            channelname = '#' + channelname
        self._chatchan = self._server.get_channel(channelname)
        self._chan = self._chatchan   # Save for IRC widget use
        schedule = eventman.get_widget_by_name('schedule')
        self._get_question_token = schedule.get_question_token
#TRANSLATORS: Future versions will need translations of this word as plugins to match the locale of the target classroom session. The word will prefix questions asked in the chatroom of the instructor
        qt = _('QUESTION:')
        qt = self._get_question_token()
        self._questioncheckbox.set_label(qt)
#TRANSLATORS: %s will be a token from the locale of the target classroom session rather than the student's locale
        tooltip =  _('"%s" directs your question to the classroom for an answer from the instructor') % qt
        self._questioncheckbox.set_tooltip_text(tooltip)

        def joined(server):
            logging.debug('connected to chatroom')
            self._input.set_sensitive(True)
            self._questioncheckbox.set_sensitive(True)
            self._textview.set_sensitive(True)
        self.event_connect_signal(self._chatchan, 'joined', joined)
        self.event_connect_signal(self._chatchan, 'message-received', self._message_received)
        self.event_connect_signal(self._chatchan, 'message-sent', self._message_sent)
        def members_changed(chan, members):
            self.emit('members-changed', members)
        self.event_connect_signal(self._chatchan, 'members-changed', members_changed)

    def do_event_disconnect(self, eventman, event):
        self._input.set_sensitive(False)
        self._questioncheckbox.set_sensitive(False)
        self._textview.set_sensitive(False)
        istart, iend = self._buffer.get_bounds()
        self._buffer.delete(istart, iend)
        self._server.disconnect()
        self.event_disconnect_signals()

    def get_input_widget(self):
        return self._input

    def _key_pressed(self, widget, event):
        if (event.keyval == gtk.keysyms.Tab):
            self._try_autocomplete()
            return True
        else:
            return False

    def _try_autocomplete(self):
        text = self._input.get_text()
        pos = self._input.get_position()

        if pos == 0 or text[pos-1] == ' ':
            return

        if len(text) > pos and text[pos] == ' ':
            space = False
        else:
            space = True

        partial_nick = text[:pos].split(' ')[-1]

        members = self._chatchan.get_members()
        nicks = filter(lambda x: partial_nick in x[:len(partial_nick)], members)

        if len(nicks) == 1:
            completion = nicks[0][len(partial_nick):]

            completion += ':'
            if space:
                completion += ' '

        elif len(nicks) > 1:
            iend = self._buffer.get_end_iter()
            nicks.sort()
            self._buffer.insert_with_tags_by_name(iend, "\n" + ' '.join(nicks), 'gray')

            self._at_bottom = (self._adjust.value == self._adjust.upper -
                               self._adjust.page_size)

            # sort by length
            nicks.sort(lambda x,y: cmp(len(x), len(y)))

            completion = ''
            for i in range(len(partial_nick), len(nicks[0])):
                # filter nicknames with the same next character
                filtered_nicks = filter(lambda x: nicks[0][i] == x[i], nicks)
                if len(nicks) == len(filtered_nicks):
                    completion += nicks[0][i]
                else:
                    break
        else: # no match
            return

        self._input.insert_text(completion, pos)
        self._input.set_position(pos + len(completion))

    def _send_text(self, *args):
        text = self._input.get_text()
        if text == '':
            return
        if text.startswith('/'):
            self._append_to_buffer(_('IRC commands are not yet supported.'))
        elif self._questioncheckbox.get_active():
            self._questioncheckbox.set_active(False)
            self._chatchan.send_message(self._get_question_token() + u' ' + unicode(text))
        else:
            self._chatchan.send_message(text)
        self._input.set_text('')

    def _message_received(self, chan, sender, text, msgtype):
        self._append_to_buffer(text, sender, msgtype)

    def _message_sent(self, chan, text):
        self._adjust.value = self._adjust.upper - self._adjust.page_size
        self._append_to_buffer(text, chan._conn._acquired_nick)

    def _scroll(self, textview, alloc):
        if self._at_bottom is None:
            self._at_bottom = True
        if self._at_bottom:
            self._adjust.value = self._adjust.upper - self._adjust.page_size



#!/usr/bin/env python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

###################### DO NOT TOUCH THIS (HEAD TO THE SECOND PART) ######################

from distutils.core import setup
import os
from DistUtilsExtra.auto import setup, install_auto

def update_data_path(path, oldvalue=None):

    try:
        fin = file('lernid/lernidconfig.py', 'r')
        fout = file(fin.name + '.new', 'w')

        for line in fin:
            fields = line.split(' = ') # Separate variable from value
            if fields[0] == '__lernid_data_directory__':
                # update to prefix, store oldvalue
                if not oldvalue:
                    oldvalue = fields[1]
                    line = "%s = '%s'\n" % (fields[0], path)
                else: # restore oldvalue
                    line = "%s = %s" % (fields[0], oldvalue)
            fout.write(line)

        fout.flush()
        fout.close()
        fin.close()
        os.rename(fout.name, fin.name)
    except (OSError, IOError), e:
        print ("ERROR: Can't find lernid/lernidconfig.py")
        sys.exit(1)
    return oldvalue


class InstallAndUpdateDataDirectory(install_auto):
    def run(self):
        if self.root or self.home:
            print "WARNING: You don't use a standard --prefix installation, take care that you eventually " \
            "need to update quickly/quicklyconfig.py file to adjust __quickly_data_directory__. You can " \
            "ignore this warning if you are packaging and uses --prefix."
        previous_value = update_data_path(os.path.join(self.prefix, 'share/lernid'))
        install_auto.run(self)
        update_data_path(self.prefix, previous_value)



##################################################################################
###################### YOU SHOULD MODIFY ONLY WHAT IS BELOW ######################
##################################################################################

setup(
    name = 'lernid',
    version = '1.0.8',
    license = 'GPL-3',
    author = 'John S Gruber',
    author_email = 'johnsgruber@gmail.com',
    description = 'Connect to Ubuntu learning events.',
    long_description = 'Lernid provides an interface for joining in Ubuntu learning events such as Ubuntu Open Week and Ubuntu Developer Week.',
    url = 'https://launchpad.net/lernid',
    data_files = [
        ('share/icons/hicolor/64x64/apps', ['data/icons/64/lernid.svg']),
        ('share/icons/hicolor/48x48/apps', ['data/icons/48/lernid.svg']),
        ('share/icons/hicolor/24x24/apps', ['data/icons/24/lernid.png']),
        ('share/icons/hicolor/22x22/apps', ['data/icons/22/lernid.png']),
        ('share/icons/hicolor/16x16/apps', ['data/icons/16/lernid.png'])
    ],
    cmdclass = {
        'install': InstallAndUpdateDataDirectory
        }
    )




#!/usr/bin/python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE


import optparse

from lernid.lernidconfig import VERSION

_instance = None

class Options(object):

    def __init__(self):
        self.option_list = []

    @classmethod
    def get_instance(cls):
        global _instance
        if not _instance:
            _instance = cls()
        return _instance

    @classmethod
    def add_option(cls, *args, **kwargs):
        opt = optparse.make_option(*args, **kwargs)
        cls.get_instance().option_list.append(opt)

    @classmethod
    def parse_args(cls):
        inst = cls.get_instance()
        parser = optparse.OptionParser(version='%prog {0}'.format(VERSION), option_list=inst.option_list)
        options, args = parser.parse_args()
        inst.options = options
        inst.args = args

    @classmethod
    def get(cls, name, default=None):
        val = getattr(cls.get_instance().options, name.replace('-', '_'))
        if val == None:
            return default
        return val




#!/usr/bin/python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gtk
import gobject
import logging
import re
import webbrowser

from lernid.widgets.Widget import Widget
from lernid.LernidOptions import Options
from urlparse import urlparse
from lernid.lernidconfig import get_data_path, VERSION
from lernid.LernidPreferences import Preferences
try:
    import webkit
    HAVE_WEBKIT = True
except ImportError:
    HAVE_WEBKIT = False

class Browser(Widget):

    __gtype_name__ = 'LernidBrowser'

    __gsignals__ = {
        'page-loading': (
            gobject.SIGNAL_RUN_LAST, None, (str,)
        ),
        'page-loaded': (
            gobject.SIGNAL_RUN_LAST, None, (str,)
        ),
        'page-changed': (
            gobject.SIGNAL_RUN_LAST, None, (str,)
        ),
    }

    embedded = HAVE_WEBKIT

    def _parse_urls(self, message):
        """Parse a message for URLs and return a tuple with the first
        item begin a URL to load or None and the second item being a
        (possibly empty) list of URLs to add to the combo box.
        """
        load = None
        ignore = []
        def cleanurl(url):
            if url.startswith('['):
                url = url[1:]
            if any([url.endswith(c) for c in '.;:]']):
                url = url[:-1]
            return url
        if re.search("slidefile", message, re.IGNORECASE): return (load, ignore)
        urls = re.findall("(\[?https?://[^\s\)]*)[\s\)\.\]]?", message, re.IGNORECASE)
        for url in urls:
            if (url.startswith('[') and url.endswith(']')) or load:
                ignore.append(cleanurl(url))
            else:
                load = cleanurl(url)
        return (load, ignore)

    if not HAVE_WEBKIT:

        def __init__(self):
            self.location = ""
            Widget.__init__(self, 'browser')

        def do_event_connect(self, eventman, event):
            classroom = eventman.get_widget_by_name('classroom')
            self.event_connect_signal(classroom, 'message-received', self._classroom_msg_received)
            #  Ideally the homepage should be opened inside Lernid itself
            self.set_location(event.homepage, False)
            schedule = eventman.get_widget_by_name('schedule')
            self._on_faculty = schedule.on_faculty # retrieve method

        def do_event_disconnect(self, eventman, event):
            self.event_disconnect_signals()

        def set_location(self, url, in_default_browser = True):
            if urlparse(url).scheme == '':
                url = 'http://' + url
            self.location = url
            logging.debug("url sent to browser " + url)
            webbrowser.open_new_tab(url)

        def get_location(self):
            return(self.location)

        def _classroom_msg_received(self, classroom, chan, sender, text):
            if not Options.get('unsafe-override') and not self._on_faculty(sender):
                return
            load, ignore = self._parse_urls(text)
            if load:
                self.set_location(load)

    else:

        def __init__(self):
            Widget.__init__(self, 'browser')

            builder = self.builder_with_file('BrowserWidget.ui')
            self.add(builder.get_object('browser_vbox'))
            self._browser = webkit.WebView()

            def sse2():
                try:
                    info_file = open("/proc/cpuinfo")
                except:
                    return False
                flag_lines = [l for l in info_file if l.startswith("flags")]
                if len(flag_lines) == 0: return False
                return "sse2" in flag_lines[0]
            
            browser_settings=webkit.WebSettings()
            useragent=browser_settings.get_property('user-agent')
            parts=useragent.split(' ')
            parts[-1]='lernid/'+VERSION
            browser_settings.set_property('user-agent', ' '.join(parts))
            if not sse2():
                browser_settings.set_property('enable-plugins', False)
            else:
                browser_settings.set_property('enable-plugins', True)
            self._browser.set_settings(browser_settings)
            
            scroll = builder.get_object('browser_scroll')
            scroll.add(self._browser)

            self._toolbar = builder.get_object('toolbar_hbox')
            self._refresh_button = builder.get_object('browser_refresh')
            self._stop_button = builder.get_object('browser_stop')

            # set up browser combo box
            self._url_combo = builder.get_object('browser_combo')
            self._url_store = gtk.ListStore(str, str, bool) # Title string, URL, delete
            self._url_combo.set_model(self._url_store)
            title_cell = gtk.CellRendererText()
            self._url_combo.pack_start(title_cell, True)
            self._url_combo.add_attribute(title_cell, 'text', 0)

            self._back_button = builder.get_object('browser_back')

            # Connect the signals
            builder.connect_signals(self)
            self._browser.connect("show", self._set_toolbar_state, True)
            self._browser.connect("hide", self._set_toolbar_state, False)
            self._browser.connect("load-started", self._set_button_state, True)
            self._browser.connect("load-finished", self._set_button_state, False)
            self._browser.connect("load-finished", self._update_url_combo)
            self._browser.set_property('can_focus', True)

            self._paused = False
            self._pause_button = builder.get_object('browser_pause')
            def toggle_paused(button):
                self._paused = not self._paused
            self._pause_button.connect('toggled', toggle_paused)

            self._toolbar.set_sensitive(False)

            self._browser_settings = self._browser.get_settings()
            self._browser_settings.set_property('user-stylesheet-uri', 'file://'+get_data_path()+'/stylesheet')

        def do_event_connect(self, eventman, event):
            classroom = eventman.get_widget_by_name('classroom')
            self.event_connect_signal(classroom, 'message-received', self._classroom_msg_received)
            self._toolbar.set_sensitive(True)
            self._browser.show()
            # FIXME: why does the button not follow sensitive state of the toolbar?
            self._pause_button.set_sensitive(True)
            #  Ideally the homepage should be opened inside Lernid itself
            self.set_location(event.homepage, False)
            schedule = eventman.get_widget_by_name('schedule')
            self._on_faculty = schedule.on_faculty # retrieve method

        def do_event_disconnect(self, eventman, event):
            self._url_combo.handler_block_by_func(self._change_page)
            self._browser.handler_block_by_func(self._update_url_combo)
            self._browser.open('about:blank')
            self._url_store.clear()
            self._browser.handler_unblock_by_func(self._update_url_combo)
            self._url_combo.handler_unblock_by_func(self._change_page)
            self._history = []
            self._current = None
            self._back_button.set_sensitive(False)
            self._pause_button.set_sensitive(False)
            self._toolbar.set_sensitive(False)
            self.event_disconnect_signals()

        def get_location(self):
            return self._browser.get_property('uri')

        def set_location(self, url, in_default_browser = True):
            if urlparse(url).scheme == '':
                    url = 'http://' + url
            logging.debug('opening url: '+url)
            # check if prefernce is set and the link was inside chat
            if Preferences.get('links_in_default_browser') and in_default_browser:
                webbrowser.open_new_tab(url)
                logging.debug('opening url in tab in default browser: '+url)
            else:
                self._browser.open(url)
                self.emit('page-changed', url)

        def _back(self, button):
            self._browser.go_back()

        def _refresh(self, button):
            self._browser.reload()

        def _stop(self, button):
            self._browser.stop_loading()

        def _set_toolbar_state(self, browser, state):
            """Make all buttons on the brower's toolbar insensitive"""
            # We make the toolbar insensitive, and thereby its children as well
            self._toolbar.set_sensitive(state)

        def _set_button_state(self, browser, frame, loading):
            """Change the stop and reload button states"""
            self._refresh_button.set_sensitive(not loading)
            self._stop_button.set_sensitive(loading)
            if loading:
                self.emit('page-loading', browser.get_property('uri'))
            if not self._browser.can_go_back():
                self._back_button.set_sensitive(False)

        def _update_url_combo(self, browser, *args):
            """Update the browser url combo box with the new title."""
            url = browser.get_property('uri')
            if url == None: 
                url = ''
            logging.debug('update url combo: {0}'.format(url))
            if url == 'about:blank':
                return
            if self._browser.can_go_back():
                self._back_button.set_sensitive(True)
            match = False
            storeiter = self._url_store.get_iter_first()
            while storeiter:
                if self._url_store.get_value(storeiter, 1) == url:
                    match = True
                    self._url_store.move_before(storeiter, self._url_store.get_iter_first())
                    self._url_combo.set_active_iter(storeiter)
                    break
                storeiter = self._url_store.iter_next(storeiter)
            if not match:
                title = browser.get_property('title')
                if title is None: title = ''
                urllen = 150 - len(title)
                shorturl = url
                if len(url) > urllen:
                    shorturl = url[:urllen] + '...'
                item = "{0} ({1})".format(title, shorturl)
                self._url_store.prepend([item, url, False])
                self._url_combo.set_active(0)
            self.emit('page-loaded', url)

        def _change_page(self, combo):
            logging.debug('change page: {0}'.format(combo.get_active()))
            if len(self._url_store) == 0:
                return
            url = self._url_store[self._url_combo.get_active()][1]
            delete = self._url_store[self._url_combo.get_active()][2]
            if delete:
                del self._url_store[self._url_combo.get_active()]
            if url != self._browser.get_property('uri'):
                logging.debug('{0} != {1}'.format(url, self._browser.get_property('uri')))
                self.set_location(url)

        def _classroom_msg_received(self, classroom, chan, sender, text):
            if not Options.get('unsafe-override') and not self._on_faculty(sender):
                return
            load, ignore = self._parse_urls(text)
            if load and not self._paused:
                self.set_location(load)
            elif load:
                ignore.insert(0, load)
            for url in ignore:
                self._url_store.prepend([url, url, True])




#!/usr/bin/python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE


import gtk
import glib

_statusbar_instance = None

class Statusbar(gtk.Statusbar):

    __gtype_name__ = 'LernidStatusbar'

    def __init__(self):
        gtk.Statusbar.__init__(self)

    @classmethod
    def get_instance(cls):
        global _statusbar_instance
        if not _statusbar_instance:
            _statusbar_instance = cls()
        return _statusbar_instance

    @classmethod
    def push_message(cls, msg, ctx='lernid', duration=None):
        obj = cls.get_instance()
        ctxid = obj.get_context_id(ctx)
        msgid = obj.push(ctxid, msg)
        if duration:
            glib.timeout_add_seconds(duration, obj.remove_message, ctxid, msgid)

    @classmethod
    def pop_message(cls, ctx='lernid'):
        obj = cls.get_instance()
        id = obj.get_context_id(ctx)
        obj.pop(id)



#!/usr/bin/python
# -*- coding: utf-8 -*-
### BEGIN LICENSE
# Copyright (C) 2009 Jono Bacon <jono@ubuntu.com>
# Copyright (C) 2010 Michael Budde <mbudde@gmail.com>
# Copyright (c) 2011 John S Gruber <johnsgruber@gmail.com>
#
#This program is free software: you can redistribute it and/or modify it
#under the terms of the GNU General Public License version 3, as published
#by the Free Software Foundation.
#
#This program is distributed in the hope that it will be useful, but
#WITHOUT ANY WARRANTY; without even the implied warranties of
#MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
#PURPOSE.  See the GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License along
#with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import gtk
import gobject
import pango
import time

from lernid.LernidPreferences import Preferences
from lernid.widgets.IrcWidget import IrcWidget

from lernid import IrcBackend
from lernid.LernidOptions import Options
from lernid.Statusbar import Statusbar

class Classroom(IrcWidget):

    __gtype_name__ = 'LernidClassroom'

    __gsignals__ = {
        'message-received': (
            gobject.SIGNAL_RUN_LAST, None, (object, str, str)
        ),
    }

    def __init__(self):
        IrcWidget.__init__(self, 'classroom')
        viewport = gtk.Viewport()
        scroll = gtk.ScrolledWindow()
        scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self._adjust = scroll.get_vadjustment()
        viewport.add(scroll)
        self.add(viewport)

        self._textview = gtk.TextView()
        self._textview.set_editable(False)
        self._textview.set_cursor_visible(False)
        self._textview.set_wrap_mode(gtk.WRAP_WORD)
        self._textview.set_left_margin(6)
        self._textview.set_right_margin(6)
        self._textview.modify_font(pango.FontDescription('Monospace'))
        self._textview.set_sensitive(False)
        self._textview.connect('size-allocate', self._scroll)
        self._at_bottom = None
        scroll.add(self._textview)

        self._buffer = self._textview.get_buffer()
        text_color = self._textview.get_style().text[gtk.STATE_INSENSITIVE]
        self._buffer.create_tag('gray', foreground_gdk=text_color)
        self._buffer.create_tag('highlight', weight=700)
        self._buffer.create_tag('italicize', style=pango.STYLE_ITALIC)
        self._init_hyperlinks()

        self.show_all()

    def do_event_connect(self, event_man, event):
        IrcWidget.do_event_connect(self, event_man, event)
        self._server = IrcBackend.Server.get_server('irc.freenode.net', event.nick, event_man)
        self._nick = event.nick
        channelname = Options.get('classroom', event.classroom)
        if len(channelname) == 0:
            return
        if channelname[0] not in '#&!+':
            channelname = '#' + channelname
        classchan = self._server.get_channel(channelname)
        self._chan = classchan  # Save for IRC widget use
        def joined(server):
            Statusbar.push_message(_('Joined classroom'),  duration=10)
            self._textview.set_sensitive(True)
        self.event_connect_signal(classchan, 'joined', joined)
        self.event_connect_signal(classchan, 'message-received', self._message_received)
        self._browser = event_man.get_widget_by_name('browser')

    def do_event_disconnect(self, event_man, event):
        self._textview.set_sensitive(False)
        istart, iend = self._buffer.get_bounds()
        self._buffer.delete(istart, iend)
        self._server.disconnect()
        self.event_disconnect_signals()

    def _message_received(self, chan, sender, text, msgtype):
        self._append_to_buffer(text, sender, msgtype)
        self.emit('message-received', chan, sender, text)

    def _scroll(self, textview, alloc):
        if self._at_bottom is None:
            self._at_bottom = True
        if self._at_bottom:
            self._adjust.value = self._adjust.upper - self._adjust.page_size

